@@ -55,7 +55,11 @@ class DocumentSelectPopUp : public QDialog
 {
 public:
     DocumentSelectPopUp(const CodeEditorBox::History & history, QWidget * parent):
-        QDialog(parent, Qt::Popup)
+    #ifndef Q_OS_MAC
+        QDialog(parent, Qt::Popup  | Qt::FramelessWindowHint)
+    #else
+        QDialog(parent, Qt::Dialog | Qt::FramelessWindowHint)
+    #endif
     {
         mModel = new QStandardItemModel(this);
         populateModel(history);
@@ -193,19 +197,99 @@ private:
     QStandardItemModel *mModel;
 };
 
+EditorTabBar::EditorTabBar(QWidget *parent):
+    QTabBar(parent)
+{
+    setDocumentMode(true);
+    setTabsClosable(true);
+    setMovable(true);
+    setUsesScrollButtons(true);
+    setDrawBase(true);
+    setElideMode(Qt::ElideNone);
+}
+
+void EditorTabBar::mousePressEvent(QMouseEvent *event)
+{
+    if (event->button() == Qt::RightButton) {
+        showContextMenu(event);
+        event->accept();
+        return;
+    }
+
+    QTabBar::mousePressEvent(event);
+}
+
+
+void EditorTabBar::mouseDoubleClickEvent(QMouseEvent *event)
+{
+    if (event->button() == Qt::LeftButton) {
+        if (tabAt(event->pos()) == -1) { // no tab under cursor
+            MainWindow::instance()->newDocument();
+            event->accept();
+            return;
+        }
+    }
+
+    QTabBar::mouseDoubleClickEvent(event);
+}
+
+void EditorTabBar::showContextMenu(QMouseEvent * event)
+{
+    mTabUnderCursor = tabAt(event->pos());
+
+    QMenu * menu = new QMenu(this);
+
+    menu->addAction(tr("Close"),                   this, SLOT(onCloseTab())           );
+    menu->addAction(tr("Close Other Tabs"),        this, SLOT(onCloseOtherTabs())     );
+    menu->addAction(tr("Close Tabs to the Right"), this, SLOT(onCloseTabsToTheRight()));
+
+    menu->popup(event->pos());
+}
+
+void EditorTabBar::onCloseTab()
+{
+    Document* doc = tabData(mTabUnderCursor).value<Document*>();
+    assert(doc);
+
+    MainWindow::close(doc);
+}
+
+void EditorTabBar::onCloseOtherTabs()
+{
+    QVector<Document*> docsToClose;
+
+    for(int currentTab = 0; currentTab != count(); ++currentTab) {
+        if (currentTab != mTabUnderCursor)
+            docsToClose.append(tabData(currentTab).value<Document*>());
+    }
+
+    for( Document * doc : docsToClose )
+        MainWindow::close(doc);
+}
+
+void EditorTabBar::onCloseTabsToTheRight()
+{
+    QVector<Document*> docsToClose;
+
+    for(int currentTab = mTabUnderCursor + 1; currentTab != count(); ++currentTab)
+        docsToClose.append(tabData(currentTab).value<Document*>());
+
+    for( Document * doc : docsToClose )
+        MainWindow::close(doc);
+}
+
+
 MultiEditor::MultiEditor( Main *main, QWidget * parent ) :
     QWidget(parent),
     mEditorSigMux(new SignalMultiplexer(this)),
     mBoxSigMux(new SignalMultiplexer(this)),
-    mDocModifiedIcon( QApplication::style()->standardIcon(QStyle::SP_DialogSaveButton) )
+#ifdef __APPLE__
+    mDocModifiedIcon( QApplication::style()->standardIcon(QStyle::SP_DriveHDIcon) )
+#else
+    mDocModifiedIcon( QIcon::fromTheme("document-save") )
+#endif
 {
-    mTabs = new QTabBar;
-    mTabs->setDocumentMode(true);
-    mTabs->setTabsClosable(true);
-    mTabs->setMovable(true);
-    mTabs->setUsesScrollButtons(true);
-    mTabs->setDrawBase(true);
-    mTabs->setElideMode(Qt::ElideNone);
+    mTabs = new EditorTabBar;
 
     CodeEditorBox *defaultBox = newBox();
 
@@ -221,14 +305,16 @@ MultiEditor::MultiEditor( Main *main, QWidget * parent ) :
 
     makeSignalConnections();
 
-    mBoxSigMux->connect(SIGNAL(currentChanged(GenericCodeEditor*)),
-                        this, SLOT(onCurrentEditorChanged(GenericCodeEditor*)));
-
     connect( &mDocModifiedSigMap, SIGNAL(mapped(QObject*)), this, SLOT(onDocModified(QObject*)) );
 
+    connect( main, SIGNAL(applySettingsRequest(Settings::Manager*)),
+             this, SLOT(applySettings(Settings::Manager*)) );
+
     createActions();
 
     setCurrentBox( defaultBox ); // will updateActions();
+
+    applySettings( main->settings() );
 }
 
 void MultiEditor::makeSignalConnections()
@@ -243,11 +329,17 @@ void MultiEditor::makeSignalConnections()
             this, SLOT(update(Document*)));
     connect(docManager, SIGNAL(showRequest(Document*, int, int)),
             this, SLOT(show(Document*, int, int)));
+    connect(docManager, SIGNAL(titleChanged(Document*)),
+            this, SLOT(update(Document*)) );
 
     connect(mTabs, SIGNAL(currentChanged(int)),
             this, SLOT(onCurrentTabChanged(int)));
     connect(mTabs, SIGNAL(tabCloseRequested(int)),
             this, SLOT(onCloseRequest(int)));
+
+    mBoxSigMux->connect(SIGNAL(currentChanged(GenericCodeEditor*)),
+            this, SLOT(onCurrentEditorChanged(GenericCodeEditor*)));
+    
 }
 
 void MultiEditor::breakSignalConnections()
@@ -255,6 +347,7 @@ void MultiEditor::breakSignalConnections()
     DocumentManager *docManager = Main::documentManager();
     docManager->disconnect(this);
     mTabs->disconnect(this);
+    mBoxSigMux->disconnect(this);
 }
 
 void MultiEditor::createActions()
@@ -375,6 +468,15 @@ void MultiEditor::createActions()
     mEditorSigMux->connect(action, SIGNAL(triggered()), SLOT(moveLineDown()));
     settings->addAction( action, "editor-move-line-down", editorCategory);
 
+    mActions[DeleteWord] = action = new QAction(
+        QIcon::fromTheme("edit-deleteword"), tr("Delete Word"), this);
+#ifdef Q_OS_MAC
+    action->setShortcut(tr("Meta+W", "Delete Word"));
+#endif
+    action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
+    mEditorSigMux->connect(action, SIGNAL(triggered()), SLOT(deleteWord()));
+    settings->addAction(action, "delete-word", editorCategory);
+
     mActions[GotoPreviousBlock] = action = new QAction(
         QIcon::fromTheme("edit-gotopreviousblock"), tr("Go to Previous Block"), this);
     action->setShortcut(tr("Ctrl+[", "Go to Previous Block"));
@@ -391,6 +493,14 @@ void MultiEditor::createActions()
                            SignalMultiplexer::ConnectionOptional);
     settings->addAction( action, "editor-go-to-next-block", editorCategory);
 
+    mActions[SelectEnclosingBlock] = action = new QAction(tr("Select Enclosing Block"), this);
+    action->setShortcut(tr("Ctrl+Shift+B", "Select Enclosing Block"));
+    action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
+    action->setStatusTip(tr("Select everything between brackets that contain cursor"));
+    mEditorSigMux->connect(action, SIGNAL(triggered()), SLOT(selectBlockAroundCursor()),
+                           SignalMultiplexer::ConnectionOptional);
+    settings->addAction( action, "editor-select-enclosing-block", editorCategory);
+
     mActions[GotoPreviousRegion] = action = new QAction(
         QIcon::fromTheme("edit-gotopreviousregion"), tr("Go to Previous Region"), this);
     action->setShortcut(tr("Ctrl+Alt+[", "Go to Previous Region"));
@@ -428,6 +538,14 @@ void MultiEditor::createActions()
 
     // View
 
+    mActions[DocClose] = action = new QAction(
+        QIcon::fromTheme("window-close"), tr("&Close"), this);
+    action->setShortcut(tr("Ctrl+W", "Close document"));
+    action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
+    action->setStatusTip(tr("Close the current document"));
+    mEditorSigMux->connect(action, SIGNAL(triggered()), SLOT(closeDocument()));
+    settings->addAction( action, "ide-document-close", editorCategory);
+
     mActions[EnlargeFont] = action = new QAction(
         QIcon::fromTheme("zoom-in"), tr("&Enlarge Font"), this);
     action->setShortcut(tr("Ctrl++", "Enlarge font"));
@@ -454,9 +572,21 @@ void MultiEditor::createActions()
     mActions[ShowWhitespace] = action = new QAction(tr("Show Spaces and Tabs"), this);
     action->setCheckable(true);
     action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
-    mEditorSigMux->connect(action, SIGNAL(triggered(bool)), SLOT(setShowWhitespace(bool)));
+    connect(action, SIGNAL(triggered(bool)), this, SLOT(setShowWhitespace(bool)));
     settings->addAction( action, "editor-toggle-show-whitespace", editorCategory);
 
+    mActions[ShowLinenumber] = action = new QAction(tr("Show Line Number"), this);
+    action->setCheckable(true);
+    action->setShortcut( tr("Ctrl+Alt+#", "Show Line Number"));
+    action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
+    connect(action, SIGNAL(triggered(bool)), this, SLOT(setShowLinenumber(bool)));
+    settings->addAction( action, "editor-toggle-show-line-number", editorCategory);
+
+    mActions[ShowAutocompleteHelp] = action = new QAction(tr("Show Autocomplete Help"), this);
+    action->setCheckable(true);
+    connect(action, SIGNAL(triggered(bool)), this, SLOT(setShowAutocompleteHelp(bool)));
+    settings->addAction( action, "editor-toggle-show-autocomplete-help", editorCategory);
+
     mActions[IndentWithSpaces] = action = new QAction(tr("Use Spaces for Indentation"), this);
     action->setCheckable(true);
     action->setStatusTip( tr("Indent with spaces instead of tabs") );
@@ -487,7 +617,7 @@ void MultiEditor::createActions()
 #ifndef Q_OS_MAC
     action->setShortcut( tr("Ctrl+Tab", "Switch Document"));
 #else
-    action->setShortcut( tr("Meta+Tab", "Switch Document"));
+    action->setShortcut( tr("Alt+Tab", "Switch Document"));
 #endif
     connect(action, SIGNAL(triggered()), this, SLOT(switchDocument()));
     settings->addAction( action, "editor-document-switch", editorCategory);
@@ -551,9 +681,11 @@ void MultiEditor::createActions()
     addAction(mActions[Cut]);
     addAction(mActions[Copy]);
     addAction(mActions[Paste]);
+    addAction(mActions[DocClose]);
     addAction(mActions[EnlargeFont]);
     addAction(mActions[ShrinkFont]);
     addAction(mActions[ShowWhitespace]);
+    addAction(mActions[ShowLinenumber]);
     addAction(mActions[IndentWithSpaces]);
     addAction(mActions[EvaluateCurrentDocument]);
     addAction(mActions[EvaluateRegion]);
@@ -564,8 +696,10 @@ void MultiEditor::createActions()
     addAction(mActions[CopyLineDown]);
     addAction(mActions[MoveLineUp]);
     addAction(mActions[MoveLineDown]);
+    addAction(mActions[DeleteWord]);
     addAction(mActions[GotoPreviousBlock]);
     addAction(mActions[GotoNextBlock]);
+    addAction(mActions[SelectEnclosingBlock]);
     addAction(mActions[GotoPreviousRegion]);
     addAction(mActions[GotoNextRegion]);
     addAction(mActions[GotoPreviousEmptyLine]);
@@ -589,13 +723,13 @@ void MultiEditor::updateActions()
     mActions[CopyLineDown]->setEnabled( editor );
     mActions[MoveLineUp]->setEnabled( editor );
     mActions[MoveLineDown]->setEnabled( editor );
+    mActions[DeleteWord]->setEnabled( editor );
     mActions[GotoPreviousEmptyLine]->setEnabled( editor );
     mActions[GotoNextEmptyLine]->setEnabled( editor );
+    mActions[DocClose]->setEnabled( editor );
     mActions[EnlargeFont]->setEnabled( editor );
     mActions[ShrinkFont]->setEnabled( editor );
     mActions[ResetFontSize]->setEnabled( editor );
-    mActions[ShowWhitespace]->setEnabled( editor );
-    mActions[ShowWhitespace]->setChecked( editor && editor->showWhitespace() );
     mActions[IndentWithSpaces]->setEnabled( scEditor );
     mActions[IndentWithSpaces]->setChecked( scEditor && scEditor->spaceIndent() );
 
@@ -604,6 +738,7 @@ void MultiEditor::updateActions()
     mActions[ToggleComment]->setEnabled( editor && editorIsScCodeEditor );
     mActions[GotoPreviousBlock]->setEnabled( editor && editorIsScCodeEditor );
     mActions[GotoNextBlock]->setEnabled( editor && editorIsScCodeEditor );
+    mActions[SelectEnclosingBlock]->setEnabled( editor && editorIsScCodeEditor );
     mActions[GotoPreviousRegion]->setEnabled( editor && editorIsScCodeEditor );
     mActions[GotoNextRegion]->setEnabled( editor && editorIsScCodeEditor );
     mActions[SelectRegion]->setEnabled( editor && editorIsScCodeEditor );
@@ -614,6 +749,16 @@ void MultiEditor::updateActions()
     mActions[EvaluateLine]->setEnabled( editor && editorIsScCodeEditor );
 }
 
+void MultiEditor::applySettings( Settings::Manager * settings )
+{
+    bool show_whitespace = settings->value("IDE/editor/showWhitespace").toBool();
+    bool show_linenumber = settings->value("IDE/editor/showLinenumber").toBool();
+    bool show_autocompletehelp = settings->value("IDE/editor/showAutocompleteHelp").toBool();
+    mActions[ShowWhitespace]->setChecked( show_whitespace );
+    mActions[ShowLinenumber]->setChecked( show_linenumber );
+    mActions[ShowAutocompleteHelp]->setChecked(show_autocompletehelp);
+}
+
 static QVariantList saveBoxState( CodeEditorBox *box, const QList<Document*> & documentList )
 {
     // save editors in reverse order - first one is last shown.
@@ -672,8 +817,10 @@ void MultiEditor::saveSession( Session *session )
     int tabCount = mTabs->count();
     for (int tabIdx = 0; tabIdx < tabCount; ++tabIdx) {
         Document *doc = documentForTab(tabIdx);
-        documentList << doc;
-        tabsData << doc->filePath();
+        if (doc) {
+            documentList << doc;
+            tabsData << doc->filePath();
+        }
     }
 
     session->setValue( "documents", QVariant::fromValue(tabsData) );
@@ -733,8 +880,8 @@ void MultiEditor::switchSession( Session *session )
 
     // close all docs
     foreach (Document *doc, documentList)
-        docManager->close(doc);
-
+    docManager->close(doc);
+    
     // remove all tabs
     while (mTabs->count())
         mTabs->removeTab(0);
@@ -754,7 +901,7 @@ void MultiEditor::switchSession( Session *session )
         QVariantList docDataList = session->value("documents").value<QVariantList>();
         foreach( const QVariant & docData, docDataList ) {
             QString filePath = docData.toString();
-            Document * doc = docManager->open(filePath, -1, 0, false);
+            Document * doc = docManager->open(filePath, -1, 0, true);
             documentList << doc;
         }
 
@@ -884,10 +1031,13 @@ void MultiEditor::onDocModified( QObject *object )
     int tabIdx = tabForDocument(doc);
     if (tabIdx == -1) return;
 
+    bool isModified = doc->textDocument()->isModified();
     QIcon icon;
-    if(doc->textDocument()->isModified())
+    if(isModified)
         icon = mDocModifiedIcon;
 
+    Main::evaluateCodeIfCompiled(QStringLiteral("Document.findByQUuid(\'%1\').prSetEdited(%2)").arg(doc->id().constData()).arg(isModified), true);
+    
     mTabs->setTabIcon( tabIdx, icon );
 }
 
@@ -906,7 +1056,7 @@ void MultiEditor::update( Document *doc )
     // update thisProcess.nowExecutingPath
     GenericCodeEditor *editor = currentEditor();
     if (editor->document() == doc)
-        Main::scProcess()->setActiveDocument(doc);
+        Main::documentManager()->setActiveDocument(doc);
 }
 
 void MultiEditor::onCloseRequest( int index )
@@ -942,7 +1092,11 @@ void MultiEditor::onBoxActivated(CodeEditorBox *box)
 
 Document * MultiEditor::documentForTab( int index )
 {
-    return mTabs->tabData(index).value<Document*>();
+    QVariant doc = mTabs->tabData(index);
+    if (doc.isValid() && !doc.isNull())
+        return doc.value<Document*>();
+    else
+        return NULL;
 }
 
 int MultiEditor::tabForDocument( Document * doc )
@@ -990,7 +1144,7 @@ void MultiEditor::setCurrentEditor( GenericCodeEditor * editor )
     updateActions();
 
     Document *currentDocument = editor ? editor->document() : 0;
-    Main::scProcess()->setActiveDocument(currentDocument);
+    Main::documentManager()->setActiveDocument(currentDocument);
     emit currentDocumentChanged(currentDocument);
 }
 
@@ -1048,4 +1202,21 @@ void MultiEditor::removeAllSplits()
     box->setFocus( Qt::OtherFocusReason );
 }
 
+void MultiEditor::setShowWhitespace(bool showWhitespace)
+{
+    Main::settings()->setValue("IDE/editor/showWhitespace", showWhitespace);
+    Main::instance()->applySettings();
+}
+
+void MultiEditor::setShowLinenumber(bool showLinenumber)
+{
+    Main::settings()->setValue("IDE/editor/showLinenumber", showLinenumber);
+    Main::instance()->applySettings();
+}
+
+void MultiEditor::setShowAutocompleteHelp(bool showAutocompleteHelp)
+{
+    Main::settings()->setValue("IDE/editor/showAutocompleteHelp", showAutocompleteHelp);
+}
+
 } // namespace ScIDE
