@@ -18,6 +18,8 @@ bcp --boost=${BOOST_ROOT} \
 	--scan external_libraries/boost*/*/*/*/*pp \
 	--scan external_libraries/threadpool/boost/*/*pp \
 	--scan external_libraries/threadpool/boost/*/*/*pp \
+	--scan external_libraries/boost_sync/*/*/*/*pp \
+	--scan external_libraries/boost_sync/*/*/*/*/*pp \
 	--scan testsuite/supernova/*pp \
 	external_libraries/boost
 
