@@ -18,46 +18,32 @@
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
-#include "OSCData.h"
 #include "PyrPrimitive.h"
 #include "PyrKernel.h"
 #include "PyrInterpreter.h"
 #include "PyrSched.h"
 #include "PyrSymbol.h"
 #include "GC.h"
-//#include "PyrOMS.h"
-//#include "MidiQ.h"
 #include <string.h>
 #include <math.h>
 #include <stdexcept>
 #include <new>
 #include <vector>
+#include <iostream>
 
-#ifdef SC_WIN32
-# include <winsock2.h>
-typedef int socklen_t;
-# define bzero( ptr, count ) memset( ptr, 0, count )
-#else
-# include <sys/socket.h>
-# include <netinet/tcp.h>
-# include <netdb.h>
-#endif
-
-#include <pthread.h>
 #include "scsynthsend.h"
 #include "sc_msg_iter.h"
+#include "SCBase.h"
+#include "server_shm.hpp"
+
 #include "SC_ComPort.h"
-#include "SC_WorldOptions.h"
-#include "SC_SndBuf.h"
 #include "SC_Endian.h"
+#include "SC_Lock.h"
+#include "SC_Msg.h"
+#include "SC_SndBuf.h"
+#include "SC_WorldOptions.h"
 
-#ifndef SC_DARWIN
-# ifndef SC_WIN32
-#  include <unistd.h>
-# endif
-#endif
-
-#include "../../../common/server_shm.hpp"
+#include <boost/asio.hpp>
 
 struct InternalSynthServerGlobals
 {
@@ -74,10 +60,9 @@ InternalSynthServerGlobals gInternalSynthServer = { 0, kNumDefaultSharedControls
 
 SC_UdpInPort* gUDPport = 0;
 
-PyrString* newPyrString(VMGlobals *g, char *s, int flags, bool collect);
+PyrString* newPyrString(VMGlobals *g, char *s, int flags, bool runGC);
 
 PyrSymbol *s_call, *s_write, *s_recvoscmsg, *s_recvoscbndl, *s_netaddr;
-const char* gPassword;
 extern bool compiledOK;
 
 std::vector<SC_UdpCustomInPort *> gCustomUdpPorts;
@@ -97,10 +82,8 @@ const int ivxNetAddr_PortID = 1;
 const int ivxNetAddr_Hostname = 2;
 const int ivxNetAddr_Socket = 3;
 
-void makeSockAddr(struct sockaddr_in &toaddr, int32 addr, int32 port);
-int sendallto(int socket, const void *msg, size_t len, struct sockaddr *toaddr, int addrlen);
-int sendall(int socket, const void *msg, size_t len);
 static int makeSynthMsgWithTags(big_scpacket *packet, PyrSlot *slots, int size);
+
 int makeSynthBundle(big_scpacket *packet, PyrSlot *slots, int size, bool useElapsed);
 
 static int addMsgSlot(big_scpacket *packet, PyrSlot *slot)
@@ -211,20 +194,6 @@ static int addMsgSlotWithTags(big_scpacket *packet, PyrSlot *slot)
 	return errNone;
 }
 
-static int makeSynthMsg(big_scpacket *packet, PyrSlot *slots, int size)
-{
-	packet->BeginMsg();
-
-	for (int i=0; i<size; ++i) {
-		int error = addMsgSlot(packet, slots+i);
-		if (error != errNone)
-			return error;
-	}
-
-	packet->EndMsg();
-	return errNone;
-}
-
 static int makeSynthMsgWithTags(big_scpacket *packet, PyrSlot *slots, int size)
 {
 	packet->BeginMsg();
@@ -262,24 +231,24 @@ static int makeSynthMsgWithTags(big_scpacket *packet, PyrSlot *slots, int size)
 }
 
 void PerformOSCBundle(int inSize, char *inData, PyrObject *inReply, int inPortNum);
-void PerformOSCMessage(int inSize, char *inData, PyrObject *inReply, int inPortNum);
-PyrObject* ConvertReplyAddress(ReplyAddress *inReply);
+void PerformOSCMessage(int inSize, char *inData, PyrObject *inReply, int inPortNum, double time);
+static PyrObject* ConvertReplyAddress(ReplyAddress *inReply);
 
-void localServerReplyFunc(struct ReplyAddress *inReplyAddr, char* inBuf, int inSize);
-void localServerReplyFunc(struct ReplyAddress *inReplyAddr, char* inBuf, int inSize)
+static void localServerReplyFunc(struct ReplyAddress *inReplyAddr, char* inBuf, int inSize)
 {
-    bool isBundle = IsBundle(inBuf);
+    double timeReceived = elapsedTime();
+	bool isBundle = IsBundle(inBuf);
 
-    pthread_mutex_lock (&gLangMutex);
+	gLangMutex.lock();
 	if (compiledOK) {
 		PyrObject *replyObj = ConvertReplyAddress(inReplyAddr);
 		if (isBundle) {
 			PerformOSCBundle(inSize, inBuf, replyObj, gUDPport->RealPortNum());
 		} else {
-			PerformOSCMessage(inSize, inBuf, replyObj, gUDPport->RealPortNum());
+			PerformOSCMessage(inSize, inBuf, replyObj, gUDPport->RealPortNum(), timeReceived);
 		}
 	}
-    pthread_mutex_unlock (&gLangMutex);
+	gLangMutex.unlock();
 
 }
 
@@ -313,30 +282,42 @@ int makeSynthBundle(big_scpacket *packet, PyrSlot *slots, int size, bool useElap
 	return errNone;
 }
 
-int netAddrSend(PyrObject *netAddrObj, int msglen, char *bufptr, bool sendMsgLen=true);
-int netAddrSend(PyrObject *netAddrObj, int msglen, char *bufptr, bool sendMsgLen)
+static int netAddrSend(PyrObject *netAddrObj, int msglen, char *bufptr, bool sendMsgLen = true)
 {
-	int err, port, addr;
+	using namespace boost::asio;
 
 	if (IsPtr(netAddrObj->slots + ivxNetAddr_Socket)) {
 		SC_TcpClientPort* comPort = (SC_TcpClientPort*)slotRawPtr(netAddrObj->slots + ivxNetAddr_Socket);
 
 		// send TCP
-		int tcpSocket = comPort->Socket();
+		ip::tcp::socket & socket = comPort->Socket();
+
+		boost::system::error_code errc;
 
 		if (sendMsgLen) {
 			// send length of message in network byte-order
-			int32 sizebuf = htonl(msglen);
-			sendall(tcpSocket, &sizebuf, sizeof(int32));
+			int32 sizebuf = sc_htonl(msglen);
+			write( socket, buffer(&sizebuf, sizeof(int32)), errc );
 		}
 
-		sendall(tcpSocket, bufptr, msglen);
+		if (!errc)
+			write( socket, buffer(bufptr, msglen), errc );
+
+		if (errc)
+		{
+			::error("Error in netAddrSend: %s\n", errc.message().c_str());
+			return errFailed;
+		}
+
+		return errNone;
 
 	} else {
 		if (gUDPport == 0) return errFailed;
 
+
 		// send UDP
-		err = slotIntVal(netAddrObj->slots + ivxNetAddr_Hostaddr, &addr);
+		int addr;
+		int err = slotIntVal(netAddrObj->slots + ivxNetAddr_Hostaddr, &addr);
 		if (err) return err;
 
 		if (addr == 0) {
@@ -348,13 +329,16 @@ int netAddrSend(PyrObject *netAddrObj, int msglen, char *bufptr, bool sendMsgLen
 			return errNone;
 		}
 
+		int port;
 		err = slotIntVal(netAddrObj->slots + ivxNetAddr_PortID, &port);
 		if (err) return err;
 
-		struct sockaddr_in toaddr;
-		makeSockAddr(toaddr, addr, port);
+		unsigned long ulAddress = (unsigned int)addr;
 
-		sendallto(gUDPport->Socket(), bufptr, msglen, (sockaddr*)&toaddr, sizeof(toaddr));
+		using namespace boost::asio;
+		ip::udp::endpoint address(ip::address_v4(ulAddress), port);
+
+		gUDPport->Socket().send_to( buffer(bufptr, msglen), address );
 	}
 
 	return errNone;
@@ -363,7 +347,7 @@ int netAddrSend(PyrObject *netAddrObj, int msglen, char *bufptr, bool sendMsgLen
 
 ///////////
 
-inline int OSCStrLen(char *str)
+inline size_t OSCStrLen(char *str)
 {
 	return (strlen(str) + 4) & ~3;
 }
@@ -371,12 +355,11 @@ inline int OSCStrLen(char *str)
 
 int makeSynthBundle(big_scpacket *packet, PyrSlot *slots, int size, bool useElapsed);
 
-static void netAddrTcpClientNotifyFunc(void *clientData);
-void netAddrTcpClientNotifyFunc(void *clientData)
+static void netAddrTcpClientNotifyFunc(void *clientData)
 {
 	extern bool compiledOK;
 
-	pthread_mutex_lock(&gLangMutex);
+	gLangMutex.lock();
 	if (compiledOK) {
 		PyrObject* netAddrObj = (PyrObject*)clientData;
 		VMGlobals* g = gMainVMGlobals;
@@ -385,11 +368,10 @@ void netAddrTcpClientNotifyFunc(void *clientData)
 		runInterpreter(g, getsym("prConnectionClosed"), 1);
 		g->canCallOS = false;
 	}
-	pthread_mutex_unlock(&gLangMutex);
+	gLangMutex.unlock();
 }
 
-int prNetAddr_Connect(VMGlobals *g, int numArgsPushed);
-int prNetAddr_Connect(VMGlobals *g, int numArgsPushed)
+static int prNetAddr_Connect(VMGlobals *g, int numArgsPushed)
 {
 	PyrSlot* netAddrSlot = g->sp;
 	PyrObject* netAddrObj = slotRawObject(netAddrSlot);
@@ -402,63 +384,35 @@ int prNetAddr_Connect(VMGlobals *g, int numArgsPushed)
 	err = slotIntVal(netAddrObj->slots + ivxNetAddr_Hostaddr, &addr);
 	if (err) return err;
 
-	struct sockaddr_in toaddr;
-	makeSockAddr(toaddr, addr, port);
-
-    int aSocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
-	if (aSocket == -1) {
-        //post("\nCould not create socket\n");
+	try {
+		SC_TcpClientPort *comPort = new SC_TcpClientPort(addr, port, netAddrTcpClientNotifyFunc, netAddrObj);
+		SetPtr(netAddrObj->slots + ivxNetAddr_Socket, comPort);
+	} catch (std::exception const & e) {
+		printf("NetAddr-Connect failed with exception: %s\n", e.what());
 		return errFailed;
 	}
 
-	const int on = 1;
-#ifdef SC_WIN32
-	if (setsockopt( aSocket, IPPROTO_TCP, TCP_NODELAY, (const char*)&on, sizeof(on)) != 0) {
-#else
-  if (setsockopt( aSocket, IPPROTO_TCP, TCP_NODELAY, &on, sizeof(on)) != 0) {
-#endif
-    //post("\nCould not setsockopt TCP_NODELAY\n");
-#ifdef SC_WIN32
-		closesocket(aSocket);
-#else
-		close(aSocket);
-#endif
-		return errFailed;
-	};
-
-
-    if(connect(aSocket,(struct sockaddr*)&toaddr,sizeof(toaddr)) != 0)
-    {
-        //post("\nCould not connect socket\n");
-#ifdef SC_WIN32
-		    closesocket(aSocket);
-#else
-    		close(aSocket);
-#endif
-        return errFailed;
-    }
-
-	SC_TcpClientPort *comPort = new SC_TcpClientPort(aSocket, netAddrTcpClientNotifyFunc, netAddrObj);
-	SetPtr(netAddrObj->slots + ivxNetAddr_Socket, comPort);
-
 	return errNone;
 }
 
-int prNetAddr_Disconnect(VMGlobals *g, int numArgsPushed);
-int prNetAddr_Disconnect(VMGlobals *g, int numArgsPushed)
+static int prNetAddr_Disconnect(VMGlobals *g, int numArgsPushed)
 {
+	int err;
+	
 	PyrSlot* netAddrSlot = g->sp;
 	PyrObject* netAddrObj = slotRawObject(netAddrSlot);
 
 	SC_TcpClientPort *comPort = (SC_TcpClientPort*)slotRawPtr(netAddrObj->slots + ivxNetAddr_Socket);
-	if (comPort) comPort->Close();
+	if (comPort) {
+		err = comPort->Close();
+		SetPtr(netAddrObj->slots + ivxNetAddr_Socket, NULL);
+	}
 
-	return errNone;
+	return err;
 }
 
 
-int prNetAddr_SendMsg(VMGlobals *g, int numArgsPushed);
-int prNetAddr_SendMsg(VMGlobals *g, int numArgsPushed)
+static int prNetAddr_SendMsg(VMGlobals *g, int numArgsPushed)
 {
 	PyrSlot* netAddrSlot = g->sp - numArgsPushed + 1;
 	PyrSlot* args = netAddrSlot + 1;
@@ -469,14 +423,11 @@ int prNetAddr_SendMsg(VMGlobals *g, int numArgsPushed)
 	if (error != errNone)
 		return error;
 
-	//for (int i=0; i<packet.size()/4; i++) post("%d %p\n", i, packet.buf[i]);
-
 	return netAddrSend(slotRawObject(netAddrSlot), packet.size(), (char*)packet.buf);
 }
 
 
-int prNetAddr_SendBundle(VMGlobals *g, int numArgsPushed);
-int prNetAddr_SendBundle(VMGlobals *g, int numArgsPushed)
+static int prNetAddr_SendBundle(VMGlobals *g, int numArgsPushed)
 {
 	PyrSlot* netAddrSlot = g->sp - numArgsPushed + 1;
 	PyrSlot* args = netAddrSlot + 1;
@@ -496,8 +447,7 @@ int prNetAddr_SendBundle(VMGlobals *g, int numArgsPushed)
 	return netAddrSend(slotRawObject(netAddrSlot), packet.size(), (char*)packet.buf);
 }
 
-int prNetAddr_SendRaw(VMGlobals *g, int numArgsPushed);
-int prNetAddr_SendRaw(VMGlobals *g, int numArgsPushed)
+static int prNetAddr_SendRaw(VMGlobals *g, int numArgsPushed)
 {
 	PyrSlot* netAddrSlot = g->sp - 1;
 	PyrSlot* arraySlot = g->sp;
@@ -515,38 +465,38 @@ int prNetAddr_SendRaw(VMGlobals *g, int numArgsPushed)
 	return netAddrSend(netAddrObj, msglen, bufptr, false);
 }
 
-int prNetAddr_GetBroadcastFlag(VMGlobals *g, int numArgsPushed);
-int prNetAddr_GetBroadcastFlag(VMGlobals *g, int numArgsPushed)
+static int prNetAddr_GetBroadcastFlag(VMGlobals *g, int numArgsPushed)
 {
-	if (gUDPport == 0) return errFailed;
-	int opt;
-	socklen_t optlen = sizeof(opt);
-#ifdef SC_WIN32
-	if (getsockopt(gUDPport->Socket(), SOL_SOCKET, SO_BROADCAST, (char *)&opt, &optlen) == -1)
-#else
-	if (getsockopt(gUDPport->Socket(), SOL_SOCKET, SO_BROADCAST, &opt, &optlen) == -1)
-#endif
-	return errFailed;
-	SetBool(g->sp, opt);
-	return errNone;
-}
-
-int prNetAddr_SetBroadcastFlag(VMGlobals *g, int numArgsPushed);
-int prNetAddr_SetBroadcastFlag(VMGlobals *g, int numArgsPushed)
-{
-	if (gUDPport == 0) return errFailed;
-	int opt = IsTrue(g->sp);
-#ifdef SC_WIN32
-	if (setsockopt(gUDPport->Socket(), SOL_SOCKET, SO_BROADCAST, (char *)&opt, sizeof(opt)) == -1)
-#else
-	if (setsockopt(gUDPport->Socket(), SOL_SOCKET, SO_BROADCAST, &opt, sizeof(opt)) == -1)
-#endif
+	if (gUDPport == 0)
 		return errFailed;
+
+	boost::system::error_code ec;
+	boost::asio::socket_base::broadcast option;
+	gUDPport->udpSocket.get_option(option, ec);
+
+	if (ec)
+		return errFailed;
+
+	SetBool(g->sp, option.value());
 	return errNone;
 }
 
-int prNetAddr_BundleSize(VMGlobals *g, int numArgsPushed);
-int prNetAddr_BundleSize(VMGlobals *g, int numArgsPushed)
+static int prNetAddr_SetBroadcastFlag(VMGlobals *g, int numArgsPushed)
+{
+	if (gUDPport == 0)
+		return errFailed;
+
+	boost::system::error_code ec;
+	boost::asio::socket_base::broadcast option(IsTrue(g->sp));
+	gUDPport->udpSocket.set_option(option, ec);
+
+	if (ec)
+		return errFailed;
+
+	return errNone;
+}
+
+static int prNetAddr_BundleSize(VMGlobals *g, int numArgsPushed)
 {
 	PyrSlot* args = g->sp;
 	big_scpacket packet;
@@ -557,8 +507,7 @@ int prNetAddr_BundleSize(VMGlobals *g, int numArgsPushed)
 	return errNone;
 }
 
-int prNetAddr_MsgSize(VMGlobals *g, int numArgsPushed);
-int prNetAddr_MsgSize(VMGlobals *g, int numArgsPushed)
+static int prNetAddr_MsgSize(VMGlobals *g, int numArgsPushed)
 {
 	PyrSlot* args = g->sp;
 	big_scpacket packet;
@@ -573,9 +522,7 @@ int prNetAddr_MsgSize(VMGlobals *g, int numArgsPushed)
 	return errNone;
 }
 
-
-int prNetAddr_UseDoubles(VMGlobals *g, int numArgsPushed);
-int prNetAddr_UseDoubles(VMGlobals *g, int numArgsPushed)
+static int prNetAddr_UseDoubles(VMGlobals *g, int numArgsPushed)
 {
 	//PyrSlot* netAddrSlot = g->sp - 1;
 	PyrSlot* flag = g->sp;
@@ -585,8 +532,7 @@ int prNetAddr_UseDoubles(VMGlobals *g, int numArgsPushed)
 	return errNone;
 }
 
-int prArray_OSCBytes(VMGlobals *g, int numArgsPushed);
-int prArray_OSCBytes(VMGlobals *g, int numArgsPushed)
+static int prArray_OSCBytes(VMGlobals *g, int numArgsPushed)
 {
 	PyrSlot* a = g->sp;
 	PyrObject *array = slotRawObject(a);
@@ -618,188 +564,194 @@ int prArray_OSCBytes(VMGlobals *g, int numArgsPushed)
 // Create a new <PyrInt8Array> object and copy data from `msg.getb'.
 // Bytes are properly untyped, but there is no <UInt8Array> type.
 
-static PyrInt8Array* MsgToInt8Array ( sc_msg_iter msg ) ;
-static PyrInt8Array* MsgToInt8Array ( sc_msg_iter msg )
+static PyrInt8Array* MsgToInt8Array ( sc_msg_iter& msg, bool runGC ) ;
+static PyrInt8Array* MsgToInt8Array ( sc_msg_iter& msg, bool runGC )
 {
 	int size = msg.getbsize() ;
 	VMGlobals *g = gMainVMGlobals ;
-	PyrInt8Array *obj = newPyrInt8Array ( g->gc , size , 0 , true ) ;
+	PyrInt8Array *obj = newPyrInt8Array ( g->gc , size , 0 , runGC ) ;
 	obj->size = size ;
 	msg.getb ( (char *)obj->b , obj->size ) ;
 	return obj ;
 }
 
-PyrObject* ConvertOSCMessage(int inSize, char *inData)
+static const double dInfinity = std::numeric_limits<double>::infinity();
+
+static PyrObject* ConvertOSCMessage(int inSize, char *inData)
 {
 	char *cmdName = inData;
 	int cmdNameLen = OSCstrlen(cmdName);
 	sc_msg_iter msg(inSize - cmdNameLen, inData + cmdNameLen);
 
 	int numElems;
-        if (inSize == cmdNameLen) {
-            numElems = 0;
-        } else {
-			if (!msg.tags) {
-				numElems = 0;
-				error("OSC messages must have type tags.  %s\n", cmdName);
-			} else {
-				numElems = strlen(msg.tags);
-			}
-        }
-        //post("tags %s %d\n", msg.tags, numElems);
+	if (inSize == cmdNameLen) {
+		numElems = 0;
+	} else {
+		if (!msg.tags) {
+			numElems = 0;
+			error("OSC messages must have type tags.  %s\n", cmdName);
+		} else {
+			numElems = strlen(msg.tags);
+		}
+	}
+	//post("tags %s %d\n", msg.tags, numElems);
 
-        VMGlobals *g = gMainVMGlobals;
-        PyrObject *obj = newPyrArray(g->gc, numElems + 1, 0, false);
-        PyrSlot *slots = obj->slots;
+	VMGlobals *g = gMainVMGlobals;
+	PyrObject *obj = newPyrArray(g->gc, numElems + 1, 0, false);
+	PyrSlot *slots = obj->slots;
 
-        SetSymbol(slots+0, getsym(cmdName));
+	SetSymbol(slots+0, getsym(cmdName));
 
-        for (int i=0; i<numElems; ++i) {
-            char tag = msg.nextTag();
-            //post("%d %c\n", i, tag);
-            switch (tag) {
-                case 'i' :
-                    SetInt(slots+i+1, msg.geti());
-                    break;
-                case 'f' :
-                    SetFloat(slots+i+1, msg.getf());
-                    break;
-                case 'd' :
-                    SetFloat(slots+i+1, msg.getd());
-                    break;
-                case 's' :
-                    SetSymbol(slots+i+1, getsym(msg.gets()));
-                    //post("sym '%s'\n", slots[i+1].us->name);
-                    break;
-                case 'b' :
-					SetObject(slots+i+1, (PyrObject*)MsgToInt8Array(msg));
-                    break;
-				case 'c':
-					SetChar(slots+i+1, (char)msg.geti());
-					break;
-				// else add the type tag as a char (jrhb 2009)
-				default:
-					SetChar(slots+i+1, tag);
-					msg.gets();
-            }
-        }
-        obj->size = numElems + 1;
-        return obj;
+	for (int i=0; i<numElems; ++i) {
+		char tag = msg.nextTag();
+		//post("%d %c\n", i, tag);
+		switch (tag) {
+		case 'i' :
+			SetInt(slots+i+1, msg.geti());
+			break;
+		case 'f' :
+			SetFloat(slots+i+1, msg.getf());
+			break;
+		case 'd' :
+			SetFloat(slots+i+1, msg.getd());
+			break;
+		case 's' :
+			SetSymbol(slots+i+1, getsym(msg.gets()));
+			//post("sym '%s'\n", slots[i+1].us->name);
+			break;
+		case 'b' : // fall through
+		case 'm' :
+			SetObject(slots+i+1, (PyrObject*)MsgToInt8Array(msg, false));
+			break;
+		case 'c':
+			SetChar(slots+i+1, (char)msg.geti());
+			break;
+		case 't' :
+			SetFloat(slots+i+1, OSCToElapsedTime(msg.gett()));
+			break;
+
+			// argument tags without any associated value
+		case 'T' :
+			SetTrue(slots+i+1);
+			msg.count ++;
+			break;
+		case 'F' :
+			SetFalse(slots+i+1);
+			msg.count ++;
+			break;
+		case 'I' :
+			SetFloat(slots+i+1, dInfinity);
+			msg.count ++;
+			break;
+		case 'N' :
+			SetNil(slots+i+1);
+			msg.count ++;
+			break;
+			// else add the type tag as a char (jrhb 2009)
+		default:
+			SetChar(slots+i+1, tag);
+			msg.gets();
+		}
+	}
+	obj->size = numElems + 1;
+	return obj;
 }
 
-PyrObject* ConvertReplyAddress(ReplyAddress *inReply)
+static PyrObject* ConvertReplyAddress(ReplyAddress *inReply)
 {
-    VMGlobals *g = gMainVMGlobals;
-    PyrObject *obj = instantiateObject(g->gc, s_netaddr->u.classobj, 2, true, false);
-    PyrSlot *slots = obj->slots;
-    SetInt(slots+0, ntohl(inReply->mSockAddr.sin_addr.s_addr));
-    SetInt(slots+1, ntohs(inReply->mSockAddr.sin_port));
-    return obj;
+	VMGlobals *g = gMainVMGlobals;
+	PyrObject *obj = instantiateObject(g->gc, s_netaddr->u.classobj, 2, true, false);
+	PyrSlot *slots = obj->slots;
+	SetInt(slots+0, inReply->mAddress.to_v4().to_ulong());
+	SetInt(slots+1, inReply->mPort);
+	return obj;
 }
 
 void PerformOSCBundle(int inSize, char* inData, PyrObject *replyObj, int inPortNum)
 {
-    // convert all data to arrays
+	// convert all data to arrays
 
-    int64 oscTime = OSCtime(inData + 8);
-    double seconds = OSCToElapsedTime(oscTime);
+	int64 oscTime = OSCtime(inData + 8);
+	double seconds = OSCToElapsedTime(oscTime);
 
-    VMGlobals *g = gMainVMGlobals;
-    ++g->sp; SetObject(g->sp, g->process);
-    ++g->sp; SetFloat(g->sp, seconds);
-    ++g->sp; SetObject(g->sp, replyObj);
-	++g->sp; SetInt(g->sp, inPortNum);
+	VMGlobals *g = gMainVMGlobals;
+	char *data = inData + 16;
+	char* dataEnd = inData + inSize;
+	while (data < dataEnd) {
+		int32 msgSize = OSCint(data);
+		data += sizeof(int32);
+		if (IsBundle(data))
+		{
+			PerformOSCBundle(msgSize, data, replyObj, inPortNum);
+		}
+		else // is a message
+		{
+			++g->sp; SetObject(g->sp, g->process);
+			++g->sp; SetFloat(g->sp, seconds);
+			++g->sp; SetObject(g->sp, replyObj);
+			++g->sp; SetInt(g->sp, inPortNum);
 
-    PyrSlot *stackBase = g->sp;
-    char *data = inData + 16;
-    char* dataEnd = inData + inSize;
-    while (data < dataEnd) {
-        int32 msgSize = OSCint(data);
-        data += sizeof(int32);
-        PyrObject *arrayObj = ConvertOSCMessage(msgSize, data);
-        ++g->sp; SetObject(g->sp, arrayObj);
-        data += msgSize;
-    }
-
-	int numMsgs = g->sp - stackBase;
-
-    runInterpreter(g, s_recvoscbndl, 4+numMsgs);
+			PyrObject *arrayObj = ConvertOSCMessage(msgSize, data);
+			++g->sp; SetObject(g->sp, arrayObj);
+			runInterpreter(g, s_recvoscmsg, 5);
+		}
+		data += msgSize;
+	}
 }
 
-void ConvertOSCBundle(int inSize, char* inData, PyrObject *replyObj)
-{
-    // convert all data to arrays
-
-    //int64 oscTime = OSCtime(inData + 8);
-    //double seconds = OSCToElapsedTime(oscTime);
-
-    VMGlobals *g = gMainVMGlobals;
-
-    int numMsgs = 0;
-    char *data = inData + 16;
-    char* dataEnd = inData + inSize;
-    while (data < dataEnd) {
-        int32 msgSize = OSCint(data);
-        data += sizeof(int32);
-        PyrObject *arrayObj = ConvertOSCMessage(msgSize, data);
-        ++g->sp; SetObject(g->sp, arrayObj);
-        numMsgs++;
-        data += msgSize;
-    }
-}
-
-void PerformOSCMessage(int inSize, char *inData, PyrObject *replyObj, int inPortNum)
+void PerformOSCMessage(int inSize, char *inData, PyrObject *replyObj, int inPortNum, double time)
 {
 
-    PyrObject *arrayObj = ConvertOSCMessage(inSize, inData);
+	PyrObject *arrayObj = ConvertOSCMessage(inSize, inData);
 
-    // call virtual machine to handle message
-    VMGlobals *g = gMainVMGlobals;
-    ++g->sp; SetObject(g->sp, g->process);
-    ++g->sp; SetFloat(g->sp, elapsedTime());	// time
-    ++g->sp; SetObject(g->sp, replyObj);
+	// call virtual machine to handle message
+	VMGlobals *g = gMainVMGlobals;
+	++g->sp; SetObject(g->sp, g->process);
+	++g->sp; SetFloat(g->sp, time);	// time
+	++g->sp; SetObject(g->sp, replyObj);
 	++g->sp; SetInt(g->sp, inPortNum);
-    ++g->sp; SetObject(g->sp, arrayObj);
-
-    runInterpreter(g, s_recvoscmsg, 5);
-
+	++g->sp; SetObject(g->sp, arrayObj);
 
+	runInterpreter(g, s_recvoscmsg, 5);
 }
 
 void FreeOSCPacket(OSC_Packet *inPacket)
 {
-    //post("->FreeOSCPacket %p\n", inPacket);
-    if (inPacket) {
-            free(inPacket->mData);
-            free(inPacket);
-    }
+	//post("->FreeOSCPacket %p\n", inPacket);
+	if (inPacket) {
+		free(inPacket->mData);
+		free(inPacket);
+	}
 }
 
-void ProcessOSCPacket(OSC_Packet* inPacket, int inPortNum)
+// takes ownership of inPacket
+void ProcessOSCPacket(OSC_Packet* inPacket, int inPortNum, double time)
 {
-    //post("recv '%s' %d\n", inPacket->mData, inPacket->mSize);
+	//post("recv '%s' %d\n", inPacket->mData, inPacket->mSize);
 	inPacket->mIsBundle = IsBundle(inPacket->mData);
 
-    pthread_mutex_lock (&gLangMutex);
+	gLangMutex.lock();
 	if (compiledOK) {
 		PyrObject *replyObj = ConvertReplyAddress(&inPacket->mReplyAddr);
 		if (compiledOK) {
 			if (inPacket->mIsBundle) {
 				PerformOSCBundle(inPacket->mSize, inPacket->mData, replyObj, inPortNum);
 			} else {
-				PerformOSCMessage(inPacket->mSize, inPacket->mData, replyObj, inPortNum);
+				PerformOSCMessage(inPacket->mSize, inPacket->mData, replyObj, inPortNum, time);
 			}
 		}
 	}
-    pthread_mutex_unlock (&gLangMutex);
+	gLangMutex.unlock();
 
-    FreeOSCPacket(inPacket);
+	FreeOSCPacket(inPacket);
 }
 
-void init_OSC(int port);
+void startAsioThread();
+void stopAsioThread();
+
 void init_OSC(int port)
 {
-    postfl("init_OSC\n");
+	postfl("init_OSC\n");
 
 #ifdef _WIN32
 	WSAData wsaData;
@@ -809,11 +761,14 @@ void init_OSC(int port)
 	}
 #endif
 
-    try {
-        gUDPport = new SC_UdpInPort(port);
-    } catch (...) {
-        postfl("No networking.");
-    }
+	startAsioThread();
+
+	try {
+		gUDPport = new SC_UdpInPort(port);
+	} catch (std::exception const & e) {
+
+		postfl("No networking: %s", e.what());
+	}
 }
 
 int prOpenUDPPort(VMGlobals *g, int numArgsPushed);
@@ -831,10 +786,10 @@ int prOpenUDPPort(VMGlobals *g, int numArgsPushed)
 		SetTrue(a);
 		newUDPport = new SC_UdpCustomInPort(port);
 		gCustomUdpPorts.push_back(newUDPport);
-    } catch (...) {
+	} catch (...) {
 		SetFalse(a);
-        postfl("Could not bind to requested port. This may mean it is in use already by another application.\n");
-    }
+		postfl("Could not bind to requested port. This may mean it is in use already by another application.\n");
+	}
 	return errNone;
 }
 
@@ -852,13 +807,17 @@ void closeAllCustomPorts()
 void cleanup_OSC()
 {
 	postfl( "cleaning up OSC\n");
+
+	stopAsioThread();
+
 #ifdef _WIN32
 	WSACleanup();
 #endif
 }
 
-int prGetHostByName(VMGlobals *g, int numArgsPushed);
-int prGetHostByName(VMGlobals *g, int numArgsPushed)
+extern boost::asio::io_service ioService;
+
+static int prGetHostByName(VMGlobals *g, int numArgsPushed)
 {
 	PyrSlot *a = g->sp;
 	char hostname[256];
@@ -866,6 +825,7 @@ int prGetHostByName(VMGlobals *g, int numArgsPushed)
 	int err = slotStrVal(a, hostname, 255);
 	if (err) return err;
 
+#if 1
 	struct hostent *he = gethostbyname(hostname);
 	if (!he) {
 #ifdef _WIN32
@@ -876,9 +836,27 @@ int prGetHostByName(VMGlobals *g, int numArgsPushed)
 		return errFailed;
 	}
 
-	SetInt(a, ntohl(*(int*)he->h_addr));
+	SetInt(a, sc_ntohl(*(int*)he->h_addr));
 
 	return errNone;
+
+#else
+	boost::asio::ip::address address;
+	boost::system::error_code err_c;
+	using boost::asio::ip::udp;
+
+	udp::resolver resolver(ioService);
+	udp::resolver::query query(hostname, "");
+	udp::resolver::iterator iterator = resolver.resolve(query, err_c);
+
+	if (err_c) {
+		error("cmp::resolver::query: %s\n", err_c.message().c_str());
+		return errFailed;
+	} else
+		SetInt(a, iterator->endpoint().address().to_v4().to_ulong() );
+
+	return errNone;
+#endif
 }
 
 int prGetLangPort(VMGlobals *g, int numArgsPushed);
@@ -890,6 +868,48 @@ int prGetLangPort(VMGlobals *g, int numArgsPushed)
 	return errNone;
 }
 
+int prMatchLangIP(VMGlobals *g, int numArgsPushed);
+int prMatchLangIP(VMGlobals *g, int numArgsPushed)
+{
+	PyrSlot *argString = g->sp;
+    char ipstring[40];
+    int err = slotStrVal(argString, ipstring, 39);
+	if (err) return err;
+    try
+    {
+        std::string loopback ("127.0.0.1");
+        // check for loopback address
+        if (!loopback.compare(ipstring)) {
+            SetTrue(g->sp - 1);
+            return errNone;
+        }
+        
+        boost::asio::io_service io_service;
+        
+        boost::asio::ip::udp::resolver resolver(io_service);
+        boost::asio::ip::udp::resolver::query query(boost::asio::ip::host_name(),"",boost::asio::ip::resolver_query_base::numeric_service);
+        boost::asio::ip::udp::resolver::iterator it=resolver.resolve(query);
+        
+        while(it!=boost::asio::ip::udp::resolver::iterator())
+        {
+            boost::asio::ip::address addr=(it++)->endpoint().address();
+            std::string candidate = addr.to_string();
+            if (!candidate.compare(ipstring)) {
+                SetTrue(g->sp - 1);
+                return errNone;
+            }
+            
+        }
+    }
+    catch(std::exception &e)
+    {
+        std::cout<<e.what()<<std::endl;
+        return errFailed;
+    }
+    SetFalse(g->sp - 1);
+	return errNone;
+}
+
 int prExit(VMGlobals *g, int numArgsPushed);
 int prExit(VMGlobals *g, int numArgsPushed)
 {
@@ -907,7 +927,6 @@ extern "C" {
 }
 
 #ifndef NO_INTERNAL_SERVER
-int prBootInProcessServer(VMGlobals *g, int numArgsPushed);
 int prBootInProcessServer(VMGlobals *g, int numArgsPushed)
 {
 	PyrSlot *a = g->sp;
@@ -967,7 +986,7 @@ int prBootInProcessServer(VMGlobals *g, int numArgsPushed)
 
 		options.mLoadGraphDefs = IsTrue(optionsSlots + 14) ? 1 : 0;
 
-		#ifdef SC_DARWIN
+		#ifdef __APPLE__
 		err = slotStrVal(optionsSlots+15, mInputStreamsEnabled, 512);
 		if(err) options.mInputStreamsEnabled = NULL;
 		else options.mInputStreamsEnabled = mInputStreamsEnabled;
@@ -1009,15 +1028,7 @@ int getScopeBuf(uint32 index, SndBuf *buf, bool& didChange)
 	}
 	return errNone;
 }
-void* wait_for_quit(void* thing);
-void* wait_for_quit(void* thing)
-{
-	World *world = (World*)thing;
-	World_WaitForQuit(world);
-	return 0;
-}
 
-int prQuitInProcessServer(VMGlobals *g, int numArgsPushed);
 int prQuitInProcessServer(VMGlobals *g, int numArgsPushed)
 {
 	//PyrSlot *a = g->sp;
@@ -1026,22 +1037,20 @@ int prQuitInProcessServer(VMGlobals *g, int numArgsPushed)
 		World *world = gInternalSynthServer.mWorld;
 		gInternalSynthServer.mWorld = 0;
 
-        pthread_t thread;
-        pthread_create(&thread, NULL, wait_for_quit, (void*)world);
-		pthread_detach(thread);
+		thread thread(std::bind(World_WaitForQuit, world, false));
+
+		thread.detach();
 	}
 
 	return errNone;
 }
-#else   // is windows
-int prQuitInProcessServer(VMGlobals *g, int numArgsPushed);
+#else // NO_INTERNAL_SERVER
 int prQuitInProcessServer(VMGlobals *g, int numArgsPushed)
 {
 	// no-op. Better to have this than to overwrite in lang.
 	return errNone;
 }
-#endif
-//#ifndef SC_WIN32
+#endif // NO_INTERNAL_SERVER
 
 inline int32 BUFMASK(int32 x)
 {
@@ -1328,6 +1337,7 @@ void init_OSC_primitives()
 	definePrimitive(base, index++, "_Array_OSCBytes", prArray_OSCBytes, 1, 0);
 	definePrimitive(base, index++, "_GetHostByName", prGetHostByName, 1, 0);
 	definePrimitive(base, index++, "_GetLangPort", prGetLangPort, 1, 0);
+    definePrimitive(base, index++, "_MatchLangIP", prMatchLangIP, 2, 0);
 	definePrimitive(base, index++, "_Exit", prExit, 1, 0);
 #ifndef NO_INTERNAL_SERVER
 	definePrimitive(base, index++, "_BootInProcessServer", prBootInProcessServer, 1, 0);
@@ -1345,7 +1355,7 @@ void init_OSC_primitives()
 	definePrimitive(base, index++, "_ServerShmInterface_getControlBusValues", prGetControlBusValues, 3, 0);
 
 	definePrimitive(base, index++, "_ServerShmInterface_setControlBusValue", prSetControlBusValue, 3, 0);
-	definePrimitive(base, index++, "_ServerShmInterface_setControlBusValues", prSetControlBusValues, 4, 0);
+	definePrimitive(base, index++, "_ServerShmInterface_setControlBusValues", prSetControlBusValues, 3, 0);
 
 	//post("initOSCRecs###############\n");
 	s_call = getsym("call");
@@ -1354,5 +1364,3 @@ void init_OSC_primitives()
 	s_recvoscbndl = getsym("recvOSCbundle");
 	s_netaddr = getsym("NetAddr");
 }
-
-
