@@ -147,21 +147,51 @@ void fatalerror(const char*str)
 
 inline int ScanSize(PyrObjectHdr *obj) { return obj->obj_format <= obj_slot ? obj->size : 0; }
 
-void PyrGC::ScanSlots(PyrSlot *inSlots, long inNumToScan)
+HOT void PyrGC::ScanSlots(PyrSlot *inSlots, long inNumToScan)
 {
+	if (inNumToScan == 0)
+		return;
+
 	unsigned char whiteColor = mWhiteColor;
+	unsigned char greyColor  = mGreyColor;
+
+	mSlotsScanned += inNumToScan;
+
+	int foundGreyObjects = 0;
+	PyrObjectHdr * grey     = &mGrey;
+	PyrObjectHdr * greyNext = grey->next;
 
 	PyrSlot *slot = inSlots;
 	PyrSlot *endslot = inSlots + inNumToScan;
-	for (; slot < endslot; ++slot) {
+	do {
 		if (IsObj(slot)) {
 			PyrObject *obj = slotRawObject(slot);
 			if (obj->gc_color == whiteColor) {
-				ToGrey2(obj);
+				/* used to be ToGrey2(obj), but rearranged for slightly better performance
+				 *
+				 * move obj from white to grey */
+
+				PyrObjectHdr * objPrev  = obj->prev;
+				PyrObjectHdr * objNext  = obj->next;
+
+				/* link in grey set */
+				greyNext->prev = obj;
+				grey->next = obj;
+				obj->prev = grey;
+				obj->next = greyNext;
+				greyNext = obj;
+
+				// remove from old set
+				objNext->prev = objPrev;
+				objPrev->next = objNext;
+
+				obj->gc_color = greyColor;
+				foundGreyObjects++;
 			}
 		}
-	}
-	mSlotsScanned += inNumToScan;
+		++slot;
+	} while (slot != endslot);
+	mNumGrey += foundGreyObjects;
 }
 
 void GCSet::Init(int inGCSet)
@@ -314,7 +344,7 @@ void PyrGC::BecomeImmutable(PyrObject *inObject)
 
 void DumpBackTrace(VMGlobals *g);
 
-PyrObject *PyrGC::New(size_t inNumBytes, long inFlags, long inFormat, bool inCollect)
+HOT PyrObject *PyrGC::New(size_t inNumBytes, long inFlags, long inFormat, bool inRunCollection)
 {
 	PyrObject *obj = NULL;
 
@@ -339,7 +369,7 @@ PyrObject *PyrGC::New(size_t inNumBytes, long inFlags, long inFormat, bool inCol
 	mNumAllocs++;
 
 	mNumToScan += credit;
-	obj = Allocate(inNumBytes, sizeclass, inCollect);
+	obj = Allocate(inNumBytes, sizeclass, inRunCollection);
 
 	obj->obj_format = inFormat;
 	obj->obj_flags = inFlags & 255;
@@ -355,7 +385,7 @@ PyrObject *PyrGC::New(size_t inNumBytes, long inFlags, long inFormat, bool inCol
 
 
 
-PyrObject *PyrGC::NewFrame(size_t inNumBytes, long inFlags, long inFormat, bool inAccount)
+HOT PyrObject *PyrGC::NewFrame(size_t inNumBytes, long inFlags, long inFormat, bool inAccount)
 {
 	PyrObject *obj = NULL;
 
@@ -390,7 +420,7 @@ PyrObject *PyrGC::NewFrame(size_t inNumBytes, long inFlags, long inFormat, bool
 	return obj;
 }
 
-PyrObject *PyrGC::NewFinalizer(ObjFuncPtr finalizeFunc, PyrObject *inObject, bool inCollect)
+PyrObject *PyrGC::NewFinalizer(ObjFuncPtr finalizeFunc, PyrObject *inObject, bool inRunCollection)
 {
 	PyrObject *obj = NULL;
 
@@ -407,7 +437,7 @@ PyrObject *PyrGC::NewFinalizer(ObjFuncPtr finalizeFunc, PyrObject *inObject, boo
 	mAllocTotal += credit;
 	mNumAllocs++;
 
-	if (inCollect && mNumToScan >= kScanThreshold) {
+	if (inRunCollection && mNumToScan >= kScanThreshold) {
 		Collect();
 	}
 
@@ -483,7 +513,7 @@ void PyrGC::CompletePartialScan(PyrObject *obj)
 	}
 }
 
-void PyrGC::DoPartialScan(int32 inObjSize)
+HOT void PyrGC::DoPartialScan(int32 inObjSize)
 {
 	int32 remain = inObjSize - mPartialScanSlot;
 	mNumPartialScans++;
@@ -508,7 +538,7 @@ void PyrGC::DoPartialScan(int32 inObjSize)
 	//post("partial %5d %2d %4d %2d %s\n", mScans, i, mNumToScan, mNumGrey, slotRawSymbol(&obj->classptr->name)->name);
 }
 
-bool PyrGC::ScanOneObj()
+HOT bool PyrGC::ScanOneObj()
 {
 	// Find a set that has a grey object
 	PyrObject* obj;
@@ -644,7 +674,7 @@ void PyrGC::Collect(int32 inNumToScan)
 	Collect();	// collect space
 }
 
-void PyrGC::Collect()
+HOT void PyrGC::Collect()
 {
 	BEGINPAUSE
 	bool stackScanned = false;
@@ -726,6 +756,31 @@ void PyrGC::ScanFinalizers()
 	}
 }
 
+void PyrGC::RunAllFinalizers()
+{
+	GCSet *gcs = &mSets[kFinalizerSet];
+
+	PyrObjectHdr *obj = gcs->mBlack.next;
+	while (!IsMarker(obj)) {
+		Finalize((PyrObject*)obj);
+		obj = obj->next;
+	}
+
+	obj = gcs->mWhite.next;
+	PyrObjectHdr *firstFreeObj = gcs->mFree;
+	while (obj != firstFreeObj) {
+		Finalize((PyrObject*)obj);
+		obj = obj->next;
+	}
+
+	obj = mGrey.next;
+	while (!IsMarker(obj)) {
+		if (obj->classptr == class_finalizer)
+			Finalize((PyrObject*)obj);
+		obj = obj->next;
+	}
+}
+
 bool PyrGC::SanityCheck2()
 {
 	int numgrey = 0;
@@ -742,10 +797,6 @@ bool PyrGC::SanityCheck2()
 	return mNumGrey == numgrey;
 }
 
-#ifdef SC_DARWIN
-	#include <CoreServices/../Frameworks/CarbonCore.framework/Headers/MacTypes.h>
-#endif
-
 bool PyrGC::SanityCheck()
 {
 	if (!mRunning) return true;
@@ -1020,7 +1071,7 @@ bool PyrGC::SanityMarkObj(PyrObject *objA, PyrObject *fromObj, int level)
 	if (objA->IsPermanent()) return true;
 	if (objA->IsMarked()) return true;
 	if (objA->size > MAXINDEXSIZE(objA)) {
-		fprintf(stderr, "obj indexed size larger than max: %d > %d\n", objA->size, MAXINDEXSIZE(objA));
+		fprintf(stderr, "obj indexed size larger than max: %d > %ld\n", objA->size, MAXINDEXSIZE(objA));
 		//dumpObject((PyrObject*)objA);
 		return false;
 	}
