@@ -1,43 +1,46 @@
 /*
 	Commandline interpreter interface.
 	Copyright (c) 2003-2006 stefan kersten.
+	Copyright (c) 2013 tim blechmann.
 
 	====================================================================
 
 	SuperCollider real time audio synthesis system
-    Copyright (c) 2002 James McCartney. All rights reserved.
+	Copyright (c) 2002 James McCartney. All rights reserved.
 	http://www.audiosynth.com
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+	You should have received a copy of the GNU General Public License
+	along with this program; if not, write to the Free Software
+	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
 #include "SC_TerminalClient.h"
+#ifdef SC_QT
+#include "../../QtCollider/LanguageClient.h"
+#endif
 
-#include <errno.h>
-#include <fcntl.h>
+#include <boost/bind.hpp>
 
-#ifdef SC_WIN32
+#ifdef _WIN32
 # define __GNU_LIBRARY__
 # include "getopt.h"
 # include "SC_Win32Utils.h"
 # include <io.h>
 # include <windows.h>
-#else
-# include <sys/param.h>
-# include <sys/poll.h>
-# include <unistd.h>
+#endif
+
+#ifdef __APPLE__
+#include "../../common/SC_Apple.hpp"
 #endif
 
 #ifdef HAVE_READLINE
@@ -46,10 +49,6 @@
 # include <signal.h>
 #endif
 
-
-#include <string.h>
-#include <time.h>
-
 #include "GC.h"
 #include "PyrKernel.h"
 #include "PyrPrimitive.h"
@@ -58,33 +57,25 @@
 #include "VMGlobals.h"
 #include "SC_DirUtils.h"   // for gIdeName
 #include "SC_LanguageConfig.hpp"
-
-#define STDIN_FD 0
+#include "SC_Version.hpp"
 
 static FILE* gPostDest = stdout;
 
-static const int ticks_per_second = 50; // every 20 milliseconds
-
 SC_TerminalClient::SC_TerminalClient(const char* name)
 	: SC_LanguageClient(name),
-	  mShouldBeRunning(false),
 	  mReturnCode(0),
 	  mUseReadline(false),
-	  mSignals(0)
-{
-	pthread_cond_init (&mCond, NULL);
-	pthread_mutex_init(&mSignalMutex, NULL);
-	pthread_mutex_init(&mInputMutex, NULL);
-	pthread_cond_init(&mInputCond, NULL);
-}
+      mWork(mIoService),
+	  mTimer(mIoService),
+#ifndef _WIN32
+	  mStdIn(mInputService, STDIN_FILENO)
+#else
+	  mStdIn(mInputService, GetStdHandle(STD_INPUT_HANDLE))
+#endif
+{}
 
 SC_TerminalClient::~SC_TerminalClient()
-{
-	pthread_cond_destroy (&mCond);
-	pthread_mutex_destroy(&mSignalMutex);
-	pthread_mutex_destroy(&mInputMutex);
-	pthread_cond_destroy(&mInputCond);
-}
+{}
 
 void SC_TerminalClient::postText(const char* str, size_t len)
 {
@@ -122,6 +113,7 @@ void SC_TerminalClient::printUsage()
 	fprintf(stdout, "Usage:\n   %s [options] [file..] [-]\n\n", getName());
 	fprintf(stdout,
 			"Options:\n"
+			"   -v                             Print supercollider version and exit\n"
 			"   -d <path>                      Set runtime directory\n"
 			"   -D                             Enter daemon mode (no input)\n"
 			"   -g <memory-growth>[km]         Set heap growth (default %s)\n"
@@ -131,7 +123,8 @@ void SC_TerminalClient::printUsage()
 			"   -r                             Call Main.run on startup\n"
 			"   -s                             Call Main.stop on shutdown\n"
 			"   -u <network-port-number>       Set UDP listening port (default %d)\n"
-			"   -i <ide-name>                  Specify IDE name (for enabling IDE-specific class code, default \"%s\")\n",
+			"   -i <ide-name>                  Specify IDE name (for enabling IDE-specific class code, default \"%s\")\n"
+			"   -a                             Standalone mode\n",
 			memGrowBuf,
 			memSpaceBuf,
 			opt.mPort,
@@ -141,7 +134,7 @@ void SC_TerminalClient::printUsage()
 
 bool SC_TerminalClient::parseOptions(int& argc, char**& argv, Options& opt)
 {
-	const char* optstr = ":d:Dg:hl:m:rsu:i:";
+	const char* optstr = ":d:Dg:hl:m:rsu:i:av";
 	int c;
 
 	// inhibit error reporting
@@ -175,6 +168,11 @@ bool SC_TerminalClient::parseOptions(int& argc, char**& argv, Options& opt)
 			case 'r':
 				opt.mCallRun = true;
 				break;
+			case 'v':
+				fprintf(stdout, "sclang %s\n", SC_VersionString().c_str());
+				quit(0);
+				return false;
+				break;
 			case 's':
 				opt.mCallStop = true;
 				break;
@@ -193,6 +191,9 @@ bool SC_TerminalClient::parseOptions(int& argc, char**& argv, Options& opt)
 			case 'i':
 				gIdeName = optarg;
 				break;
+			case 'a':
+				opt.mStandalone = true;
+				break;
 			default:
 				::post("%s: unknown error (getopt)\n", getName());
 				quit(255);
@@ -249,15 +250,13 @@ int SC_TerminalClient::run(int argc, char** argv)
 	// read library configuration file
 	if (opt.mLibraryConfigFile)
 		SC_LanguageConfig::setConfigFile(opt.mLibraryConfigFile);
-
-	SC_LanguageConfig::readLibraryConfig();
+	SC_LanguageConfig::readLibraryConfig(opt.mStandalone);
 
 	// initialize runtime
 	initRuntime(opt);
 
 	// startup library
-	mShouldBeRunning = true;
-	compileLibrary();
+	compileLibrary(opt.mStandalone);
 
 	// enter main loop
 	if (codeFile) executeFile(codeFile);
@@ -268,8 +267,8 @@ int SC_TerminalClient::run(int argc, char** argv)
 	}
 	else {
 		initInput();
-		if( shouldBeRunning() ) startInput();
-		if( shouldBeRunning() ) commandLoop();
+		startInput();
+		commandLoop();
 		endInput();
 		cleanupInput();
 	}
@@ -285,36 +284,40 @@ int SC_TerminalClient::run(int argc, char** argv)
 	return mReturnCode;
 }
 
+void SC_TerminalClient::recompileLibrary()
+{
+    SC_LanguageClient::recompileLibrary(mOptions.mStandalone);
+}
+
 void SC_TerminalClient::quit(int code)
 {
 	mReturnCode = code;
-	mShouldBeRunning = false;
 }
 
-void SC_TerminalClient::interpretCmdLine(PyrSymbol* method, SC_StringBuffer& cmdLine)
+static PyrSymbol * resolveMethodSymbol(bool silent)
 {
-	setCmdLine(cmdLine);
-	cmdLine.reset();
-	runLibrary(method);
-	flush();
+	if (silent)
+		return s_interpretCmdLine;
+	else
+		return s_interpretPrintCmdLine;
 }
 
-void SC_TerminalClient::interpretCmdLine(PyrSymbol* method, const char* cmdLine)
+void SC_TerminalClient::interpretCmdLine(const char* cmdLine, bool silent)
 {
 	setCmdLine(cmdLine);
-	runLibrary(method);
+	runLibrary(resolveMethodSymbol(silent));
 	flush();
 }
 
 
-void SC_TerminalClient::interpretCmdLine(PyrSymbol* method, const char *cmdLine, size_t size)
+void SC_TerminalClient::interpretCmdLine(const char *cmdLine, size_t size, bool silent)
 {
 	setCmdLine(cmdLine, size);
-	runLibrary(method);
+	runLibrary(resolveMethodSymbol(silent));
 	flush();
 }
 
-// WARNING: Call with input locked!
+// Note: called only if the input thread does not perform an asynchronous read operation
 void SC_TerminalClient::interpretInput()
 {
 	char *data = mInputBuf.getData();
@@ -323,10 +326,10 @@ void SC_TerminalClient::interpretInput()
 	while( i < c ) {
 		switch (data[i]) {
 		case kInterpretCmdLine:
-			interpretCmdLine(s_interpretCmdLine, data, i);
+			interpretCmdLine(data, i, true);
 			break;
 		case kInterpretPrintCmdLine:
-			interpretCmdLine(s_interpretPrintCmdLine, data, i);
+			interpretCmdLine(data, i, false);
 			break;
 
 		case kRecompileLibrary:
@@ -343,7 +346,11 @@ void SC_TerminalClient::interpretInput()
 		i = 0;
 	}
 	mInputBuf.reset();
-	if( mUseReadline ) pthread_cond_signal( &mInputCond );
+
+	if (mUseReadline)
+		mReadlineSem.post();
+	else
+		startInputRead();
 }
 
 void SC_TerminalClient::onLibraryStartup()
@@ -360,83 +367,57 @@ void SC_TerminalClient::onLibraryStartup()
 
 void SC_TerminalClient::sendSignal( Signal sig )
 {
-	lockSignal();
-	mSignals |= sig;
-	pthread_cond_signal( &mCond );
-	unlockSignal();
+	switch (sig) {
+	case sig_input:
+		mIoService.post( boost::bind(&SC_TerminalClient::interpretInput, this) );
+		break;
+
+	case sig_recompile:
+		mIoService.post( boost::bind(&SC_TerminalClient::recompileLibrary, this) );
+		break;
+
+	case sig_sched:
+		mIoService.post( boost::bind(&SC_TerminalClient::tick, this, boost::system::error_code()) );
+		break;
+
+	case sig_stop:
+		mIoService.post( boost::bind(&SC_TerminalClient::stopMain, this) );
+		break;
+	}
 }
 
 void SC_TerminalClient::onQuit( int exitCode )
 {
-	lockSignal();
 	postfl("main: quit request %i\n", exitCode);
 	quit( exitCode );
-	pthread_cond_signal( &mCond );
-	unlockSignal();
+	stop();
 }
 
-extern void ElapsedTimeToTimespec(double elapsed, struct timespec *spec);
+extern void ElapsedTimeToChrono(double elapsed, std::chrono::system_clock::time_point & out_time_point);
+
+void SC_TerminalClient::tick( const boost::system::error_code& error )
+{
+	mTimer.cancel();
+
+	double secs;
+	lock();
+	bool haveNext = tickLocked( &secs );
+	unlock();
+
+	flush();
+
+	std::chrono::system_clock::time_point nextAbsTime;
+	ElapsedTimeToChrono( secs, nextAbsTime );
+
+	if (haveNext) {
+		mTimer.expires_at(nextAbsTime);
+		mTimer.async_wait(boost::bind(&SC_TerminalClient::tick, this, _1));
+	}
+}
 
 void SC_TerminalClient::commandLoop()
 {
-	bool haveNext = false;
-	struct timespec nextAbsTime;
-
-	lockSignal();
-
-	while( shouldBeRunning() )
-	{
-
-		while ( mSignals ) {
-			int sig = mSignals;
-			mSignals = 0;
-
-			unlockSignal();
-
-			if (sig & sig_input) {
-				//postfl("input\n");
-				lockInput();
-				interpretInput();
-				unlockInput();
-			}
-
-			if (sig & sig_sched) {
-				//postfl("tick\n");
-				double secs;
-				lock();
-				haveNext = tickLocked( &secs );
-				unlock();
-
-				flush();
-
-				//postfl("tick -> next time = %f\n", haveNext ? secs : -1);
-				ElapsedTimeToTimespec( secs, &nextAbsTime );
-			}
-
-			if (sig & sig_stop) {
-				stopMain();
-			}
-
-			if (sig & sig_recompile) {
-				recompileLibrary();
-			}
-
-			lockSignal();
-		}
-
-		if( !shouldBeRunning() ) {
-			break;
-		}
-		else if( haveNext ) {
-			int result = pthread_cond_timedwait( &mCond, &mSignalMutex, &nextAbsTime );
-			if( result == ETIMEDOUT ) mSignals |= sig_sched;
-		}
-		else {
-			pthread_cond_wait( &mCond, &mSignalMutex );
-		}
-	}
-
-	unlockSignal();
+	mIoService.run();
 }
 
 void SC_TerminalClient::daemonLoop()
@@ -495,20 +476,15 @@ void SC_TerminalClient::readlineCmdLine( char *cmdLine )
 		return;
 	}
 
-	if(*cmdLine!=0){
+	if( *cmdLine != 0 ) {
 		// If line wasn't empty, store it so that uparrow retrieves it
 		add_history(cmdLine);
 		int len = strlen(cmdLine);
 
-		client->lockInput();
 		client->mInputBuf.append(cmdLine, len);
 		client->mInputBuf.append(kInterpretPrintCmdLine);
 		client->sendSignal(sig_input);
-		// Wait for input to be processed,
-		// so that its output is displayed before readline prompt.
-		if (client->mInputShouldBeRunning)
-			pthread_cond_wait( &client->mInputCond, &client->mInputMutex );
-		client->unlockInput();
+		client->mReadlineSem.wait();
 	}
 }
 
@@ -535,221 +511,108 @@ void SC_TerminalClient::readlineInit()
 #endif
 }
 
-#ifndef _WIN32
-
-void *SC_TerminalClient::readlineFunc( void *arg )
-{
-	readlineInit();
-
-	SC_TerminalClient *client = static_cast<SC_TerminalClient*>(arg);
-
-	fd_set fds;
-	FD_ZERO(&fds);
-
-	while(true) {
-		FD_SET(STDIN_FD, &fds);
-		FD_SET(client->mInputCtlPipe[0], &fds);
-
-		if( select(FD_SETSIZE, &fds, NULL, NULL, NULL) < 0 ) {
-			if( errno == EINTR ) continue;
-			postfl("readline: select() error:\n%s\n", strerror(errno));
-			client->onQuit(1);
-			break;
-		}
-
-		if( FD_ISSET(client->mInputCtlPipe[0], &fds) ) {
-			postfl("readline: quit requested\n");
-			break;
-		}
-
-		if( FD_ISSET(STDIN_FD, &fds) ) {
-			rl_callback_read_char();
-		}
-	}
-
-	postfl("readline: stopped.\n");
-
-	return NULL;
-}
-
-#else
-
-void *SC_TerminalClient::readlineFunc( void *arg )
-{
-	readlineInit();
-
-	SC_TerminalClient *client = static_cast<SC_TerminalClient*>(arg);
-
-	HANDLE hStdIn = GetStdHandle(STD_INPUT_HANDLE);
-	HANDLE hnds[] = { client->mQuitInputEvent, hStdIn };
-
-	bool shouldRun = true;
-	while (shouldRun) {
-		DWORD result = WaitForMultipleObjects( 2, hnds, false, INFINITE );
-
-		if( result == WAIT_FAILED ) {
-			postfl("readline: wait error.\n");
-			client->onQuit(1);
-			break;
-		}
-
-		int hIndex = result - WAIT_OBJECT_0;
-
-		if( hIndex == 0 ) {
-			postfl("readline: quit requested.\n");
-			break;
-		}
-
-		if( hIndex == 1 ) {
-			rl_callback_read_char();
-		}
-	}
-
-	postfl("readline: stopped.\n");
-
-	return NULL;
-}
-#endif // !_WIN32
-
 #endif // HAVE_READLINE
 
+void SC_TerminalClient::startInputRead()
+{
 #ifndef _WIN32
-
-void *SC_TerminalClient::pipeFunc( void *arg )
-{
-	SC_TerminalClient *client = static_cast<SC_TerminalClient*>(arg);
-
-	ssize_t bytes;
-	const size_t toRead = 256;
-	char buf[toRead];
-	SC_StringBuffer stack;
-
-	fd_set fds;
-	FD_ZERO(&fds);
-
-	bool shouldRead = true;
-	while(shouldRead) {
-		FD_SET(STDIN_FD, &fds);
-		FD_SET(client->mInputCtlPipe[0], &fds);
-
-		if( select(FD_SETSIZE, &fds, NULL, NULL, NULL) < 0 ) {
-			if( errno == EINTR ) continue;
-			postfl("pipe-in: select() error: %s\n", strerror(errno));
-			client->onQuit(1);
-			break;
-		}
-
-		if( FD_ISSET(client->mInputCtlPipe[0], &fds) ) {
-			postfl("pipe-in: quit requested\n");
-			break;
-		}
-
-		if( FD_ISSET(STDIN_FD, &fds) ) {
-
-			while(true) {
-				bytes = read( STDIN_FD, buf, toRead );
-
-				if( bytes > 0 ) {
-					client->pushCmdLine( stack, buf, bytes );
-				}
-				else if( bytes == 0 ) {
-					postfl("pipe-in: EOF. Will quit.\n");
-					client->onQuit(0);
-					shouldRead = false;
-					break;
-				}
-				else {
-					if( errno == EAGAIN ) {
-						break; // no more to read this time;
-					}
-					else if( errno != EINTR ){
-						postfl("pipe-in: read() error: %s\n", strerror(errno));
-						client->onQuit(1);
-						shouldRead = false;
-						break;
-					}
-				}
-			}
-		}
-	}
-
-	postfl("pipe-in: stopped.\n");
-
-	return NULL;
-}
-
+	if (mUseReadline)
+		mStdIn.async_read_some(boost::asio::null_buffers(), boost::bind(&SC_TerminalClient::onInputRead, this, _1, _2));
+	else
+		mStdIn.async_read_some(boost::asio::buffer(inputBuffer), boost::bind(&SC_TerminalClient::onInputRead, this, _1, _2));
 #else
-
-void *SC_TerminalClient::pipeFunc( void *arg )
-{
-	SC_TerminalClient *client = static_cast<SC_TerminalClient*>(arg);
-	SC_StringBuffer stack;
-	HANDLE hStdIn = GetStdHandle(STD_INPUT_HANDLE);
-	char buf[256];
-	while(1) {
-		DWORD n;
-		BOOL ok = ReadFile( hStdIn, &buf, 256, &n, NULL );
-		if(ok) {
-			client->pushCmdLine( stack, buf, n );
-		}
+	mStdIn.async_wait( [&] (const boost::system::error_code & error) {
+		if(error)
+			onInputRead(error, 0);
 		else {
-			postfl("pipe-in: ERROR (ReadFile): %i\n", GetLastError());
-			client->onQuit(1);
-			break;
+			DWORD bytes_transferred;
+
+			::ReadFile(GetStdHandle(STD_INPUT_HANDLE),
+					   inputBuffer.data(),
+					   inputBuffer.size(),
+					   &bytes_transferred,
+					   nullptr);
+
+			onInputRead(error, bytes_transferred);
 		}
-	}
-	return NULL;
+	});
+#endif
 }
 
+void SC_TerminalClient::onInputRead(const boost::system::error_code &error, std::size_t bytes_transferred)
+{
+	if (error == boost::asio::error::operation_aborted) {
+		postfl("SCLang Input: Quit requested\n");
+		return;
+	}
+
+	if (error == boost::asio::error::eof) {
+		postfl("SCLang Input: EOF. Will quit.\n");
+		onQuit(0);
+		return;
+	}
+
+	if (error) {
+		postfl("SCLang Input: %s.\n", error.message().c_str());
+		onQuit(1);
+		return;
+	}
+
+	if (!error) {
+#if HAVE_READLINE
+		if (mUseReadline) {
+			rl_callback_read_char();
+			startInputRead();
+			return;
+		}
+#endif
+		pushCmdLine( inputBuffer.data(), bytes_transferred );
+	}
+}
+
+void SC_TerminalClient::inputThreadFn()
+{
+#if HAVE_READLINE
+	if (mUseReadline)
+		readlineInit();
 #endif
 
-void SC_TerminalClient::pushCmdLine( SC_StringBuffer &buf, const char *newData, size_t size)
+	startInputRead();
+
+	boost::asio::io_service::work work(mInputService);
+	mInputService.run();
+}
+
+
+void SC_TerminalClient::pushCmdLine( const char *newData, size_t size)
 {
-	lockInput();
-
 	bool signal = false;
-
 	while (size--) {
 		char c = *newData++;
 		switch (c) {
 		case kRecompileLibrary:
 		case kInterpretCmdLine:
 		case kInterpretPrintCmdLine:
-			mInputBuf.append( buf.getData(), buf.getSize() );
+			mInputBuf.append( mInputThrdBuf.getData(), mInputThrdBuf.getSize() );
 			mInputBuf.append(c);
 			signal = true;
-			buf.reset();
+			mInputThrdBuf.reset();
 			break;
 
 		default:
-			buf.append(c);
+			mInputThrdBuf.append(c);
 		}
 	}
 
-	if(signal) sendSignal(sig_input);
-
-	unlockInput();
+	if (signal)
+		sendSignal(sig_input);
+	else
+		startInputRead();
 }
 
 
-
-
 void SC_TerminalClient::initInput()
 {
-
-#ifndef _WIN32
-	if( pipe( mInputCtlPipe ) == -1 ) {
-		postfl("Error creating pipe for input thread control:\n%s\n", strerror(errno));
-		quit(1);
-	}
-#else
-	mQuitInputEvent = CreateEvent( NULL, false, false, NULL );
-	if( mQuitInputEvent == NULL ) {
-		postfl("Error creating event for input thread control.\n");
-		quit(1);
-	}
-#endif
-
 #ifdef HAVE_READLINE
 	if (strcmp(gIdeName, "none") == 0) {
 		// Other clients (emacs, vim, ...) won't want to interact through rl
@@ -757,57 +620,20 @@ void SC_TerminalClient::initInput()
 		return;
 	}
 #endif
-
-#ifndef _WIN32
-	if( fcntl( STDIN_FD, F_SETFL, O_NONBLOCK ) == -1 ) {
-		postfl("Error setting up non-blocking pipe reading:\n%s\n", strerror(errno));
-		quit(1);
-	}
-#endif // !_WIN32
 }
 
 
 void SC_TerminalClient::startInput()
 {
-	mInputShouldBeRunning = true;
-#ifdef HAVE_READLINE
-	if( mUseReadline )
-		pthread_create( &mInputThread, NULL, &SC_TerminalClient::readlineFunc, this );
-	else
-#endif
-		pthread_create( &mInputThread, NULL, &SC_TerminalClient::pipeFunc, this );
+	thread thread(std::bind(&SC_TerminalClient::inputThreadFn, this));
+	mInputThread = std::move(thread);
 }
 
 void SC_TerminalClient::endInput()
 {
-	// NOTE: On Windows, there is no way to safely interrupt
-	// the pipe-reading thread. So just quit and let it die.
-
-#ifdef _WIN32
-	if (mUseReadline) {
-#endif
-	// wake up the input thread in case it is waiting
-	// for input to be processed
-	lockInput();
-		mInputShouldBeRunning = false;
-		pthread_cond_signal( &mInputCond );
-	unlockInput();
-
-#ifndef _WIN32
-	char c = 'q';
-	ssize_t bytes = write( mInputCtlPipe[1], &c, 1 );
-	if( bytes < 1 )
-		postfl("WARNING: could not send quit command to input thread.\n");
-#else
-	SetEvent( mQuitInputEvent );
-#endif
-
+	mInputService.stop();
 	postfl("main: waiting for input thread to join...\n");
-	pthread_join( mInputThread, NULL );
-
-#ifdef _WIN32
-	} // if (mUseReadline)
-#endif
+	mInputThread.join();
 	postfl("main: quitting...\n");
 }
 
@@ -832,7 +658,7 @@ int SC_TerminalClient::prArgv(struct VMGlobals* g, int)
 		PyrString* str = newPyrString(g->gc, argv[i], 0, true);
 		SetObject(argvObj->slots+i, str);
 		argvObj->size++;
-		g->gc->GCWrite(argvObj, (PyrObject*)str);
+		g->gc->GCWriteNew(argvObj, (PyrObject*)str); // we know str is white so we can use GCWriteNew
 	}
 
 	return errNone;
@@ -861,4 +687,24 @@ int SC_TerminalClient::prRecompile(struct VMGlobals *, int)
 	static_cast<SC_TerminalClient*>(instance())->sendSignal(sig_recompile);
 	return errNone;
 }
-// EOF
+
+SCLANG_DLLEXPORT SC_LanguageClient * createLanguageClient(const char * name)
+{
+	if (SC_LanguageClient::instance())
+		return NULL;
+
+#ifdef __APPLE__
+	SC::Apple::disableAppNap();
+#endif
+
+#ifdef SC_QT
+	return new QtCollider::LangClient(name);
+#else
+	return new SC_TerminalClient(name);
+#endif
+}
+
+SCLANG_DLLEXPORT void destroyLanguageClient(class SC_LanguageClient * languageClient)
+{
+	delete languageClient;
+}
