@@ -192,6 +192,13 @@ struct Dreset : public Unit
 	float prev_reset;
 };
 
+struct Dconst : public Unit
+{
+  float m_total;
+  float m_runningsum;
+  float m_tolerance;
+};
+
 extern "C"
 {
 
@@ -279,6 +286,9 @@ void Dreset_next(Dreset *unit, int inNumSamples);
 void Donce_Ctor(Donce *unit);
 void Donce_next(Donce *unit, int inNumSamples);
 
+void Dconst_Ctor(Dconst *unit);
+void Dconst_next(Dconst *unit, int inNumSamples);
+
 };
 
 
@@ -433,22 +443,28 @@ void Demand_Ctor(Demand *unit)
 		}
 	}
 
-	unit->m_prevout = (float*) RTAlloc(unit->mWorld, unit->mNumOutputs * sizeof(float));
-	unit->m_out = (float**) RTAlloc(unit->mWorld, unit->mNumOutputs * sizeof(float*));
+	for (int i=0; i<unit->mNumOutputs; ++i)
+		OUT0(i) = 0.f;
+
+	char * memoryChunk = (char*)RTAlloc(unit->mWorld, unit->mNumOutputs * (sizeof(float) + sizeof(float*)));
+
+	if (!memoryChunk) {
+		Print("Demand: RT memory allocation failed\n");
+		SETCALC(ClearUnitOutputs);
+		return;
+	}
+
+	unit->m_prevout = (float*)memoryChunk;
+	unit->m_out = (float**)(memoryChunk + unit->mNumOutputs * sizeof(float));
 
-	//Print("Demand_Ctor calc %p\n", unit->mCalcFunc);
 	unit->m_prevtrig = 0.f;
 	unit->m_prevreset = 0.f;
-	for (int i=0; i<unit->mNumOutputs; ++i) {
-		unit->m_prevout[i] = 0.f;
-		OUT0(i) = 0.f;
-	}
+	std::fill_n(unit->m_prevout, unit->mNumOutputs, 0.f);
 }
 
 void Demand_Dtor(Demand* unit)
 {
 	if(unit->m_prevout) RTFree(unit->mWorld, unit->m_prevout);
-	if(unit->m_out)     RTFree(unit->mWorld, unit->m_out);
 }
 
 
@@ -499,10 +515,9 @@ void Duty_next_da(Duty *unit, int inNumSamples)
 			out[i] = x;
 
 		} else {
-			count--;
 			out[i] = prevout;
 		}
-
+		count--;
 		prevreset = zreset;
 	}
 
@@ -723,13 +738,16 @@ void DemandEnvGen_next_k(DemandEnvGen *unit, int inNumSamples)
 
 
 				// new shape
+				float fshape = DEMANDINPUT(d_env_shape);
+				if (sc_isnan(fshape))
+					shape = unit->m_shape;
+				else
+					shape = (int)fshape;
+
+				curve = DEMANDINPUT(d_env_curve);
+				if (sc_isnan(curve)) curve = unit->m_curve;
 
 				float count;
-				shape = (int)DEMANDINPUT(d_env_shape);
-				curve = DEMANDINPUT(d_env_curve);
-
-				if (sc_isnan(shape)) shape = unit->m_shape;
-				if (sc_isnan(curve)) curve = unit->m_curve;
 				if (phase <= 1.f) {
 					shape = 1; // shape_Linear
 					count = 1.f;
@@ -982,11 +1000,15 @@ void DemandEnvGen_next_a(DemandEnvGen *unit, int inNumSamples)
 				// new shape
 				float count;
 				curve = DEMANDINPUT_A(d_env_curve, i + 1);
-				shape = (int)DEMANDINPUT_A(d_env_shape, i + 1);
 
 				// printf("shapes: %i \n", shape);
 				if (sc_isnan(curve)) curve = unit->m_curve;
-				if (sc_isnan(shape)) shape = unit->m_shape;
+
+				float fshape = DEMANDINPUT_A(d_env_shape, i + 1);
+				if (sc_isnan(fshape))
+					shape = unit->m_shape;
+				else
+					shape = (int)fshape;
 
 				if (phase <= 1.f) {
 					shape = 1; // shape_Linear
@@ -1914,19 +1936,22 @@ void Dshuf_scramble(Dshuf *unit) {
 
 void Dshuf_Ctor(Dshuf *unit)
 {
-	int32 i, size;
-
-	size = (int32)(unit->mNumInputs) - 1;
+	OUT0(0) = 0.f;
 
+	uint32 size = (unit->mNumInputs) - 1;
 	unit->m_indices = (int32*)RTAlloc(unit->mWorld, size * sizeof(int32));
 
-	for(i=0; i < size; ++i) {
-		unit->m_indices[i] = i + 1;
+	if (!unit->m_indices) {
+		Print("Dshuf: RT memory allocation failed\n");
+		SETCALC(ClearUnitOutputs);
+		return;
 	}
 
+	for(uint32 i=0; i < size; ++i)
+		unit->m_indices[i] = i + 1;
+
 	SETCALC(Dshuf_next);
 	Dshuf_next(unit, 0);
-	OUT0(0) = 0.f;
 }
 
 void Dshuf_Dtor(Dshuf *unit)
@@ -2047,6 +2072,68 @@ void Dstutter_Ctor(Dstutter *unit)
 
 //////////////////////////////
 
+void Dconst_next(Dconst *unit, int inNumSamples)
+{
+	float total, tolerance;
+
+	if (inNumSamples) {
+		if(sc_isnan(unit->m_runningsum)) {
+			OUT0(0) = NAN;
+			return;
+		}
+
+		if (unit->m_total < 0.f) {
+
+			total = DEMANDINPUT_A(0, inNumSamples);
+			tolerance = DEMANDINPUT_A(2, inNumSamples);
+			if(sc_isnan(total) || sc_isnan(tolerance)) {
+				OUT0(0) = NAN;
+				return;
+			}
+			unit->m_total = total;
+			unit->m_tolerance = tolerance;
+
+		} {
+			total = unit->m_total;
+			tolerance = unit->m_tolerance;
+		}
+
+		float val = DEMANDINPUT_A(1, inNumSamples);
+
+		if(sc_isnan(val)) {
+			OUT0(0) = NAN;
+			return;
+		} else {
+			float runningsum = unit->m_runningsum + val;
+			if((runningsum > total) || (fabs(total - runningsum) <= tolerance)) {
+				OUT0(0) = total - unit->m_runningsum;
+				unit->m_runningsum = NAN;	// stop next time
+			} else {
+				unit->m_runningsum = runningsum;
+				OUT0(0) = val;
+			}
+		}
+
+	} else {
+
+		unit->m_total = -1.f;
+		unit->m_runningsum = 0.f;
+		RESETINPUT(0);
+		RESETINPUT(1);
+		RESETINPUT(2);
+
+	}
+}
+
+void Dconst_Ctor(Dconst *unit)
+{
+	SETCALC(Dconst_next);
+	Dconst_next(unit, 0);
+	OUT0(0) = 0.f;
+}
+
+//////////////////////////////
+
 
 
 void Dpoll_next(Dpoll *unit, int inNumSamples)
@@ -2066,13 +2153,23 @@ void Dpoll_next(Dpoll *unit, int inNumSamples)
 
 void Dpoll_Ctor(Dpoll *unit)
 {
-	SETCALC(Dpoll_next);
-	unit->m_id = IN0(3); // number of chars in the id string
-	unit->m_id_string = (char*)RTAlloc(unit->mWorld, ((int)unit->m_id + 1) * sizeof(char));
-	for(int i = 0; i < (int)unit->m_id; i++) {
-		unit->m_id_string[i] = (char)IN0(4+i);
+	OUT0(0) = 0.f;
+
+	const int idStringSize = (int)IN0(3);
+
+	unit->m_id_string = (char*)RTAlloc(unit->mWorld, (idStringSize + 1) * sizeof(char));
+
+	if (!unit->m_id_string) {
+		Print("Dpoll: RT memory allocation failed\n");
+		SETCALC(ClearUnitOutputs);
+		return;
 	}
-	unit->m_id_string[(int)unit->m_id] = '\0';
+
+	for(int i = 0; i < idStringSize; i++)
+		unit->m_id_string[i] = (char)IN0(4+i);
+
+	SETCALC(Dpoll_next);
+	unit->m_id_string[idStringSize] = '\0';
 	unit->m_mayprint = unit->mWorld->mVerbosity >= -1;
 	OUT0(0) = 0.f;
 }
@@ -2169,8 +2266,13 @@ inline double sc_loop(Unit *unit, double in, double hi, int loop)
 
 #define D_GET_BUF \
 	float fbufnum  = DEMANDINPUT_A(0, inNumSamples);; \
+	if (sc_isnan(fbufnum)) { \
+		OUT0(0) = NAN; \
+		return; \
+	} \
+	fbufnum = sc_max(0.f, fbufnum); \
 	if (fbufnum != unit->m_fbufnum) { \
-		uint32 bufnum = (int)fbufnum; \
+		int bufnum = (int)fbufnum; \
 		World *world = unit->mWorld; \
 		if (bufnum < 0) { bufnum = 0; } \
 		if (bufnum >= world->mNumSndBufs) { \
@@ -2198,8 +2300,13 @@ inline double sc_loop(Unit *unit, double in, double hi, int loop)
 
 #define D_GET_BUF_SHARED \
 	float fbufnum  = DEMANDINPUT_A(0, inNumSamples);; \
+	if (sc_isnan(fbufnum)) { \
+		OUT0(0) = NAN; \
+		return; \
+	} \
+	fbufnum = sc_max(0.f, fbufnum); \
 	if (fbufnum != unit->m_fbufnum) { \
-		uint32 bufnum = (int)fbufnum; \
+		int bufnum = (int)fbufnum; \
 		World *world = unit->mWorld; \
 		if (bufnum < 0) { bufnum = 0; } \
 		if (bufnum >= world->mNumSndBufs) { \
@@ -2346,6 +2453,7 @@ PluginLoad(Demand)
 	DefineSimpleUnit(Dswitch1);
 	DefineSimpleUnit(Dswitch);
 	DefineSimpleUnit(Dstutter);
+	DefineSimpleUnit(Dconst);
 	DefineSimpleUnit(Donce);
 	DefineSimpleUnit(Dreset);
 	DefineDtorUnit(Dpoll);
