@@ -28,11 +28,7 @@
 #ifdef NOVA_SIMD
 #include "simd_memory.hpp"
 
-#if defined(__GNUC__) && !defined(__clang__)
-#define inline_functions __attribute__ ((flatten))
-#else
-#define inline_functions
-#endif
+#include "function_attributes.h"
 
 #endif
 
@@ -73,7 +69,7 @@ struct Poll : public Unit
 {
 	int m_samplesRemain, m_intervalSamples;
 	float m_trig;
-	float m_lastPoll, m_id;
+	float m_lastPoll;
 	char *m_id_string;
 	bool m_mayprint;
 };
@@ -374,8 +370,8 @@ extern "C"
 ////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 #ifdef NOVA_SIMD
-inline_functions void Trig1_next_nova(Trig1 *unit, int inNumSamples);
-inline_functions void Trig1_next_k_nova(Trig1 *unit, int inNumSamples);
+FLATTEN void Trig1_next_nova(Trig1 *unit, int inNumSamples);
+FLATTEN void Trig1_next_k_nova(Trig1 *unit, int inNumSamples);
 #endif
 
 void Trig1_Ctor(Trig1 *unit)
@@ -398,7 +394,7 @@ void Trig1_Ctor(Trig1 *unit)
 	unit->mCounter = 0;
 	unit->m_prevtrig = 0.f;
 
-	ZOUT0(0) = 0.f;
+	Trig1_next(unit, 1);
 }
 
 void Trig1_next(Trig1 *unit, int inNumSamples)
@@ -578,7 +574,7 @@ void Trig_Ctor(Trig *unit)
 	unit->m_prevtrig = 0.f;
 	unit->mLevel = 0.f;
 
-	ZOUT0(0) = 0.f;
+	Trig_next(unit, 1);
 }
 
 void Trig_next(Trig *unit, int inNumSamples)
@@ -768,14 +764,15 @@ void SendTrig_next(SendTrig *unit, int inNumSamples)
 void SendTrig_next_aka(SendTrig *unit, int inNumSamples)
 {
 	float *trig = ZIN(0);
-        float *value = ZIN(2);
+	int id = (int)ZIN0(1);
+	float *value = ZIN(2);
 	float prevtrig = unit->m_prevtrig;
 
 	LOOP1(inNumSamples,
 		float curtrig = ZXP(trig);
-                float curval = ZXP(value);
+		float curval = ZXP(value);
 		if (curtrig > 0.f && prevtrig <= 0.f) {
-			SendTrigger(&unit->mParent->mNode, (int)ZIN0(1), curval);
+			SendTrigger(&unit->mParent->mNode, id, curval);
 		}
 		prevtrig = curtrig;
 	);
@@ -785,6 +782,9 @@ void SendTrig_next_aka(SendTrig *unit, int inNumSamples)
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////////
 
+static void Unit_next_nop(SendReply * unit, int inNumSamples)
+{}
+
 void SendReply_Ctor(SendReply *unit)
 {
 	const int kVarOffset = 3;
@@ -792,28 +792,36 @@ void SendReply_Ctor(SendReply *unit)
 	unit->m_prevtrig = 0.f;
 	unit->m_cmdNameSize = IN0(2);
 	unit->m_valueSize = unit->mNumInputs - unit->m_cmdNameSize - kVarOffset;
+	unit->m_valueOffset = kVarOffset + unit->m_cmdNameSize;
 
 	// allocations
-	unit->m_cmdName = (char*)RTAlloc(unit->mWorld, (unit->m_cmdNameSize + 1) * sizeof(char));
-	for(int i = 0; i < (int)unit->m_cmdNameSize; i++){
+	const int cmdNameAllocSize = (unit->m_cmdNameSize + 1) * sizeof(char);
+	const int valuesAllocSize = unit->m_valueSize * sizeof(float);
+	char * chunk = (char*)RTAlloc(unit->mWorld,cmdNameAllocSize + valuesAllocSize);
+
+	if (!chunk) {
+		Print("SendReply: RT memory allocation failed\n");
+		SETCALC(Unit_next_nop);
+		return;
+	}
+
+	unit->m_cmdName = chunk;
+	unit->m_values  = (float*)(chunk + cmdNameAllocSize);
+
+	for(int i = 0; i < (int)unit->m_cmdNameSize; i++)
 		unit->m_cmdName[i] = (char)IN0(kVarOffset+i);
-	};
+
 	// terminate string
 	unit->m_cmdName[unit->m_cmdNameSize] = 0;
 
-	unit->m_valueOffset = kVarOffset + unit->m_cmdNameSize;
-	unit->m_values = (float*)RTAlloc(unit->mWorld, unit->m_valueSize * sizeof(float));
-
-	if (INRATE(0) == calc_FullRate) {
+	if (INRATE(0) == calc_FullRate)
 		SETCALC(SendReply_next_aka);
-	} else {
+	else
 		SETCALC(SendReply_next);
-	}
 }
 
 void SendReply_Dtor(SendReply* unit)
 {
-	RTFree(unit->mWorld, unit->m_values);
 	RTFree(unit->mWorld, unit->m_cmdName);
 }
 
@@ -828,9 +836,9 @@ void SendReply_next(SendReply *unit, int inNumSamples)
 	for(int j = 0; j < inNumSamples; j++) {
 		float curtrig = trig[j];
 		if (curtrig > 0.f && prevtrig <= 0.f) {
-			for(int i=0; i<valueSize; i++) {
+			for (int i=0; i<valueSize; i++)
 				values[i] = IN(i + valueOffset)[0];
-			}
+
 			SendNodeReply(&unit->mParent->mNode, (int)ZIN0(1), unit->m_cmdName, unit->m_valueSize, values);
 		}
 		prevtrig = curtrig;
@@ -842,14 +850,13 @@ void SendReply_next_aka(SendReply *unit, int inNumSamples)
 {
 	float *trig = IN(0);
 	float prevtrig = unit->m_prevtrig;
-	float *invalues = IN(unit->m_valueOffset);
 	float *values = unit->m_values;
 	int valueSize = unit->m_valueSize;
 	int valueOffset = unit->m_valueOffset;
 	for(int j = 0; j < inNumSamples; j++) {
 		float curtrig = trig[j];
 		if (curtrig > 0.f && prevtrig <= 0.f) {
-			for(int i=0; i<valueSize; i++) {
+			for (int i=0; i<valueSize; i++) {
 				int offset = INRATE( i + valueOffset ) != calc_FullRate ? 0 : j;
 				values[i] = IN(i + valueOffset)[offset];
 			}
@@ -868,22 +875,28 @@ void Poll_Ctor(Poll* unit)
 	if (INRATE(0) == calc_FullRate){
 		if (INRATE(1) == calc_FullRate){
 			SETCALC(Poll_next_aa);
-			} else {
-			SETCALC(Poll_next_ak);
-			}
 		} else {
-		SETCALC(Poll_next_kk);
+			SETCALC(Poll_next_ak);
 		}
+	} else {
+		SETCALC(Poll_next_kk);
+	}
 
 	unit->m_trig = IN0(0);
-	unit->m_id = IN0(3); // number of chars in the id string
-	unit->m_id_string = (char*)RTAlloc(unit->mWorld, ((int)unit->m_id + 1) * sizeof(char));
-	for(int i = 0; i < (int)unit->m_id; i++){
-		unit->m_id_string[i] = (char)IN0(4+i);
-		};
-	unit->m_id_string[(int)unit->m_id] = '\0';
+	const int idSize = (int)IN0(3); // number of chars in the id string
+	unit->m_id_string = (char*)RTAlloc(unit->mWorld, (idSize + 1) * sizeof(char));
 
-	unit->m_mayprint = unit->mWorld->mVerbosity >= 0;
+	if (!unit->m_id_string) {
+		Print("Poll: RT memory allocation failed\n");
+		SETCALC(Unit_next_nop);
+		return;
+	}
+
+	for(int i = 0; i < idSize; i++)
+		unit->m_id_string[i] = (char)IN0(4+i);
+
+	unit->m_id_string[idSize] = '\0';
+	unit->m_mayprint = unit->mWorld->mVerbosity >= -1;
 
 	Poll_next_kk(unit, 1);
 }
@@ -893,17 +906,19 @@ void Poll_Dtor(Poll* unit)
 	RTFree(unit->mWorld, unit->m_id_string);
 }
 
-void Poll_next_aa(Poll *unit, int inNumSamples){
+void Poll_next_aa(Poll *unit, int inNumSamples)
+{
 	float* in = IN(1);
 	float* trig = IN(0);
 	float lasttrig = unit->m_trig;
-	for(int i = 0; i < inNumSamples; i++){
+	for (int i = 0; i < inNumSamples; i++){
 		if((lasttrig <= 0.0) && (trig[i] > 0.0)){
-			if(unit->m_mayprint){
+			if (unit->m_mayprint)
 				Print("%s: %g\n", unit->m_id_string, in[i]);
-			}
-			if(IN0(2) >= 0.0) SendTrigger(&unit->mParent->mNode, (int)IN0(2), in[i]);
-			}
+
+			if (IN0(2) >= 0.0)
+				SendTrigger(&unit->mParent->mNode, (int)IN0(2), in[i]);
+		}
 		lasttrig = trig[i];
 	}
 	unit->m_trig = lasttrig;
@@ -913,10 +928,11 @@ void Poll_next_kk(Poll *unit, int inNumSamples){
 	float in = IN0(1);
 	float trig = IN0(0);
 	if((unit->m_trig <= 0.0) && (trig > 0.0)){
-		if(unit->m_mayprint){
+		if(unit->m_mayprint)
 			Print("%s: %g\n", unit->m_id_string, in);
-		}
-		if(IN0(2) >= 0.0) SendTrigger(&unit->mParent->mNode, (int)IN0(2), in);
+
+		if(IN0(2) >= 0.0)
+			SendTrigger(&unit->mParent->mNode, (int)IN0(2), in);
 	}
 	unit->m_trig = trig;
 }
@@ -954,7 +970,7 @@ void SetResetFF_Ctor(SetResetFF *unit)
 	unit->m_prevreset = 0.f;
 	unit->mLevel = 0.f;
 
-	ZOUT0(0) = 0.f;
+	SetResetFF_next_k(unit, 1);
 }
 
 
@@ -990,10 +1006,14 @@ void SetResetFF_next_k(SetResetFF *unit, int inNumSamples)
 	float prevreset = unit->m_prevreset;
 	float level = unit->mLevel;
 
+	float curtrig = ZXP(trig);
 	if (prevreset <= 0.f && curreset > 0.f) level = 0.f;
+	else if (prevtrig <= 0.f && curtrig > 0.f) level = 1.f;
+	ZXP(out) = level;
+	prevtrig = curtrig;
 
-	LOOP1(inNumSamples,
-		float curtrig = ZXP(trig);
+	LOOP(inNumSamples - 1,
+		curtrig = ZXP(trig);
 		if (prevtrig <= 0.f && curtrig > 0.f) level = 1.f;
 		ZXP(out) = level;
 		prevtrig = curtrig;
@@ -1126,7 +1146,7 @@ void Latch_next_aa(Latch *unit, int inNumSamples)
 ////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 #ifdef NOVA_SIMD
-inline_functions void Gate_next_ak_nova(Gate *unit, int inNumSamples)
+FLATTEN void Gate_next_ak_nova(Gate *unit, int inNumSamples)
 {
 	float *trig = ZIN(1);
 	float level = unit->mLevel;
@@ -1139,7 +1159,7 @@ inline_functions void Gate_next_ak_nova(Gate *unit, int inNumSamples)
 		nova::setvec_simd(OUT(0), level, inNumSamples);
 }
 
-inline_functions void Gate_next_ak_nova_64(Gate *unit, int inNumSamples)
+FLATTEN void Gate_next_ak_nova_64(Gate *unit, int inNumSamples)
 {
 	float *trig = ZIN(1);
 	float level = unit->mLevel;
@@ -1170,7 +1190,7 @@ void Gate_Ctor(Gate *unit)
 
 	unit->mLevel = 0.f;
 
-	ZOUT0(0) = 0.f;
+	Gate_next_ak(unit, 1);
 }
 
 
@@ -1221,7 +1241,7 @@ void Schmidt_Ctor(Schmidt *unit)
 
 	unit->mLevel = 0.f;
 
-	ZOUT0(0) = 0.f;
+	Schmidt_next(unit, 1);
 }
 
 void Schmidt_next(Schmidt *unit, int inNumSamples)
@@ -1253,7 +1273,7 @@ void PulseDivider_Ctor(PulseDivider *unit)
 	unit->mLevel = 0.f;
 	unit->mCounter = (long)floor(ZIN0(2) + 0.5);
 
-	ZOUT0(0) = 0.f;
+	PulseDivider_next(unit, 1);
 }
 
 
@@ -1302,7 +1322,7 @@ void PulseCount_Ctor(PulseCount *unit)
 	unit->m_prevreset = 0.f;
 	unit->mLevel = 0.f;
 
-	ZOUT0(0) = 0.f;
+	PulseCount_next_k(unit, 1);
 }
 
 
@@ -1393,7 +1413,7 @@ void Stepper_Ctor(Stepper *unit)
 	unit->m_prevreset = 0.f;
 	unit->mLevel = (float)resetval;
 
-	ZOUT0(0) = 0.f;
+	Stepper_next_ak(unit, 1);
 }
 
 
@@ -1487,7 +1507,7 @@ void TDelay_Ctor(TDelay *unit)
 	unit->m_prevtrig = 0.f;
 	unit->mCounter = 0;
 
-	ZOUT0(0) = 0.f;
+	TDelay_next(unit, 1);
 }
 
 
@@ -1874,8 +1894,8 @@ void Peak_next_ak_unroll(Peak *unit, int inNumSamples);
 void Peak_next_ai_unroll(Peak *unit, int inNumSamples);
 
 #ifdef NOVA_SIMD
-inline_functions void Peak_next_ak_k_nova(Peak *unit, int inNumSamples);
-inline_functions void Peak_next_ai_k_nova(Peak *unit, int inNumSamples);
+FLATTEN void Peak_next_ak_k_nova(Peak *unit, int inNumSamples);
+FLATTEN void Peak_next_ai_k_nova(Peak *unit, int inNumSamples);
 #endif
 
 void Peak_Ctor(Peak *unit)
@@ -2814,6 +2834,11 @@ struct SendPeakRMS:
 		size_t cmdNameAllocSize = (cmdNameSize + 1) * sizeof(char);
 
 		void * allocData = RTAlloc(unit->mWorld, channelDataAllocSize + cmdNameAllocSize);
+		if (!allocData) {
+			Print("SendPeakRMS: RT memory allocation failed\n");
+			SETCALC(Unit_next_nop);
+			return;
+		}
 
 		memset(allocData, 0, channelDataAllocSize);
 		mChannelData = (float*)allocData;
