@@ -27,6 +27,9 @@
 #include "SC_HiddenWorld.h"
 #include "SC_Prototypes.h"
 #include "SC_StringParser.h"
+#include "SC_WorldOptions.h"
+#include "SC_TimeDLL.hpp"
+#include "SC_Time.hpp"
 
 #include <jack/jack.h>
 #include <math.h>
@@ -42,101 +45,17 @@
 static const char* kJackDriverIdent = "JackDriver";
 static const char* kJackDefaultClientName = "SuperCollider";
 
-// =====================================================================
-// Timing
-
-// =====================================================================
-// SC_JackDLL
-//
-//   Delay-Locked-Loop after
-//   Fons Adriaensen, "Using a DLL to filter time"
-
-#define SC_JACK_DLL_BW 0.012
-
-class SC_JackDLL
-{
-public:
-	SC_JackDLL()
-		: m_b(0.), m_c(0.),
-		  m_t0(0.), m_t1(0.), m_e2(0.),
-		  m_np(0), m_ei(0.), m_ec(0)
-	{ }
-
-	void Reset(double sampleRate, jack_nframes_t periodFrames, double bandWidth, double t)
-	{
-		// compute coefficients
-		m_np = periodFrames;
-		m_b = 2*M_PI*bandWidth*m_np/sampleRate;
-		m_c = m_b*m_b/2.;
-		// initialize filter
-		double tp = m_np/sampleRate;
-		m_e2 = tp;
-		m_t0 = t;
-		m_t1 = t + tp;
-		// initialize statistics
-		m_ei = 0.;
-		m_ec = 0;
-	}
-
-	void Update(double t)
-	{
-		// compute error
-		double e = m_e = t - m_t1;
-		// update filter
-		m_t0 = m_t1;
-		m_t1 += m_b * e + m_e2;
-		m_e2 += m_c * e;
-		// collect statistics
-		m_ei += e;
-		m_ec++;
-	}
-
-	double PeriodTime() const { return m_t0; }
-	double NextPeriodTime() const { return m_t1; }
-	double Period() const { return m_t1 - m_t0; }
-	double SampleRate() const { return m_np/Period(); }
-	double Error() const { return m_e; }
-	double AvgError() const { return m_ec > 0 ? m_ei/m_ec : 0; }
-
-private:
-	double m_b, m_c;
-	double m_t0, m_t1, m_e, m_e2;
-	jack_nframes_t m_np;
-	double m_ei;
-	int m_ec;
-};
-
-static inline int64 sc_JackOSCTime(const struct timeval& tv)
-{
-	return ((int64)(tv.tv_sec + kSECONDS_FROM_1900_to_1970) << 32)
-		+ (int64)(tv.tv_usec * kMicrosToOSCunits);
-}
-
-static inline int64 sc_JackOSCTime()
-{
-	struct timeval tv;
-	gettimeofday(&tv, 0);
-	return sc_JackOSCTime(tv);
-}
-
-static inline double sc_JackOSCTimeSeconds()
-{
-	return (uint64)sc_JackOSCTime() * kOSCtoSecs;
-}
-
 int32 server_timeseed()
 {
-	static int32 count = 0;
-	struct timeval tv;
-	gettimeofday(&tv, 0);
-	return (int32)tv.tv_sec ^ (int32)tv.tv_usec ^ count--;
+	return timeSeed();
 }
 
 int64 oscTimeNow()
 {
-	return sc_JackOSCTime();
+	return OSCTime(getTime());
 }
 
+
 void initializeScheduler()
 {
 }
@@ -164,24 +83,24 @@ struct SC_JackPortList
 
 class SC_JackDriver : public SC_AudioDriver
 {
-    jack_client_t		*mClient;
+	jack_client_t		*mClient;
 	SC_JackPortList		*mInputList;
 	SC_JackPortList		*mOutputList;
 	double				mMaxOutputLatency;
-	SC_JackDLL			mDLL;
+	SC_TimeDLL			mDLL;
 
 protected:
-    // driver interface methods
+	// driver interface methods
 	virtual bool DriverSetup(int* outNumSamplesPerCallback, double* outSampleRate);
 	virtual bool DriverStart();
 	virtual bool DriverStop();
 
 public:
-    SC_JackDriver(struct World *inWorld);
+	SC_JackDriver(struct World *inWorld);
 	virtual ~SC_JackDriver();
 
 	// process loop
-    void Run();
+	void Run();
 
 	// reset state
 	void Reset(double sampleRate, int bufferSize);
@@ -208,7 +127,7 @@ private:
 
 SC_AudioDriver* SC_NewAudioDriver(struct World *inWorld)
 {
-    return new SC_JackDriver(inWorld);
+	return new SC_JackDriver(inWorld);
 }
 
 // =====================================================================
@@ -273,7 +192,7 @@ void sc_jack_shutdown_cb(void* arg)
 	scprintf("%s: killed by jack\n", kJackDriverIdent);
 	World * world = (World*)arg;
 	world->hw->mTerminating = true;
-	world->hw->mQuitProgram->Release();
+	world->hw->mQuitProgram->post();
 }
 
 // =====================================================================
@@ -281,10 +200,10 @@ void sc_jack_shutdown_cb(void* arg)
 
 SC_JackDriver::SC_JackDriver(struct World *inWorld)
 	: SC_AudioDriver(inWorld),
-	  mClient(0),
-	  mInputList(0),
-	  mOutputList(0),
-	  mMaxOutputLatency(0.)
+	mClient(0),
+	mInputList(0),
+	mOutputList(0),
+	mMaxOutputLatency(0.)
 { }
 
 SC_JackDriver::~SC_JackDriver()
@@ -373,48 +292,48 @@ bool SC_JackDriver::DriverSetup(int* outNumSamples, double* outSampleRate)
 
 void SC_JackDriver::ConnectClientInputs(const char * pattern)
 {
-    const char **ports = jack_get_ports (mClient, pattern, NULL, JackPortIsOutput);
-    jack_port_t ** ourports = mInputList->mPorts;
+	const char **ports = jack_get_ports (mClient, pattern, NULL, JackPortIsOutput);
+	jack_port_t ** ourports = mInputList->mPorts;
 
-    if (!ports)
-        return;
+	if (!ports)
+		return;
 
-    int i = 0;
-    while (ports[i]) {
-        if (i == mInputList->mSize)
-            break;
+	int i = 0;
+	while (ports[i]) {
+		if (i == mInputList->mSize)
+			break;
 
-        const char * src = ports[i];
-        const char * dst = jack_port_name(ourports[i]);
+		const char * src = ports[i];
+		const char * dst = jack_port_name(ourports[i]);
 
-        ConnectPorts(src, dst);
-        ++i;
-    }
+		ConnectPorts(src, dst);
+		++i;
+	}
 
-    free(ports);
+	free(ports);
 }
 
 void SC_JackDriver::ConnectClientOutputs(const char * pattern)
 {
-    const char **ports = jack_get_ports (mClient, pattern, NULL, JackPortIsInput);
-    jack_port_t ** ourports = mOutputList->mPorts;
+	const char **ports = jack_get_ports (mClient, pattern, NULL, JackPortIsInput);
+	jack_port_t ** ourports = mOutputList->mPorts;
 
-    if (!ports)
-        return;
+	if (!ports)
+		return;
 
-    int i = 0;
-    while (ports[i]) {
-        if (i == mOutputList->mSize)
-            break;
+	int i = 0;
+	while (ports[i]) {
+		if (i == mOutputList->mSize)
+			break;
 
-        const char * src = jack_port_name(ourports[i]);
-        const char * dst = ports[i];
+		const char * src = jack_port_name(ourports[i]);
+		const char * dst = ports[i];
 
-        ConnectPorts(src, dst);
-        ++i;
-    }
+		ConnectPorts(src, dst);
+		++i;
+	}
 
-    free(ports);
+	free(ports);
 }
 
 bool SC_JackDriver::DriverStart()
@@ -476,14 +395,15 @@ bool SC_JackDriver::DriverStop()
 	if (mClient) err = jack_deactivate(mClient);
 	return err == 0;
 }
-
+void sc_SetDenormalFlags();
 void SC_JackDriver::Run()
 {
+    sc_SetDenormalFlags();
 	jack_client_t* client = mClient;
 	World* world = mWorld;
 
-#if SC_JACK_USE_DLL
-	mDLL.Update(sc_JackOSCTimeSeconds());
+#ifdef SC_JACK_USE_DLL
+	mDLL.Update(secondsSinceEpoch(getTime()));
 #if SC_JACK_DEBUG_DLL
 	static int tick = 0;
 	if (++tick >= 10) {
@@ -494,10 +414,9 @@ void SC_JackDriver::Run()
 	}
 #endif
 #else
-	struct timeval hostTime;
-	gettimeofday(&hostTime, 0);
+	HostTime hostTime = getTime();
 
-	double hostSecs = (double)hostTime.tv_sec + (double)hostTime.tv_usec * 1e-6;
+	double hostSecs = secondsSinceEpoch(hostTime);
 	double sampleTime = (double)(jack_frame_time(client) + jack_frames_since_cycle_start(client));
 
 	if (mStartHostSecs == 0) {
@@ -559,14 +478,14 @@ void SC_JackDriver::Run()
 		}
 
 		// main loop
-#if SC_JACK_USE_DLL
+#ifdef SC_JACK_USE_DLL
 		int64 oscTime = mOSCbuftime = (int64)((mDLL.PeriodTime() - mMaxOutputLatency) * kSecondsToOSCunits + .5);
 // 		int64 oscInc = mOSCincrement = (int64)(mOSCincrementNumerator / mDLL.SampleRate());
 		int64 oscInc = mOSCincrement = (int64)((mDLL.Period() / numBufs) * kSecondsToOSCunits + .5);
 		mSmoothSampleRate = mDLL.SampleRate();
 		double oscToSamples = mOSCtoSamples = mSmoothSampleRate * kOSCtoSecs /* 1/pow(2,32) */;
 #else
-		int64 oscTime = mOSCbuftime = sc_JackOSCTime(hostTime) - (int64)(mMaxOutputLatency * kSecondsToOSCunits + .5);
+		int64 oscTime = mOSCbuftime = OSCTime(hostTime) - (int64)(mMaxOutputLatency * kSecondsToOSCunits + .5);
 		int64 oscInc = mOSCincrement;
 		double oscToSamples = mOSCtoSamples;
 #endif
@@ -652,12 +571,12 @@ void SC_JackDriver::Reset(double sampleRate, int bufferSize)
 	mMaxPeakCounter = (int)mBuffersPerSecond;
 	mOSCincrement = (int64)(mOSCincrementNumerator / mSampleRate);
 
-#if SC_JACK_USE_DLL
+#ifdef SC_JACK_USE_DLL
 	mDLL.Reset(
 		mSampleRate,
 		mNumSamplesPerCallback,
-		SC_JACK_DLL_BW,
-		sc_JackOSCTimeSeconds());
+		SC_TIME_DLL_BW,
+		secondsSinceEpoch(getTime()));
 #endif
 }
 
@@ -679,7 +598,9 @@ bool SC_JackDriver::GraphOrderChanged()
 	jack_nframes_t lat = 0;
 
 	for (int i=0; i < outputs->mSize; ++i) {
-		jack_nframes_t portLat = jack_port_get_total_latency(mClient, outputs->mPorts[i]);
+		jack_latency_range_t range;
+		jack_port_get_latency_range( outputs->mPorts[i], JackPlaybackLatency, &range );
+		jack_nframes_t portLat = range.max;
 		if (portLat > lat) lat = portLat;
 	}
 
