@@ -20,7 +20,6 @@
 
 
 #include "SC_Lib.h"
-#include "SC_ComPort.h"
 #include "SC_CoreAudio.h"
 #include "SC_HiddenWorld.h"
 #include "SC_Graph.h"
@@ -33,6 +32,7 @@
 #include <new>
 #include "SC_Prototypes.h"
 #include "scsynthsend.h"
+#include "SC_WorldOptions.h"
 
 extern int gMissingNodeID;
 
@@ -776,7 +776,9 @@ SCErr meth_d_free(World *inWorld, int inSize, char *inData, ReplyAddress *inRepl
 	while (msg.remain()) {
 		int32* defname = msg.gets4();
 		if (!defname) return kSCErr_SynthDefNotFound;
-		GraphDef_Remove(inWorld, defname);
+		SCErr err = GraphDef_Remove(inWorld, defname);
+        if(err != kSCErr_None)
+            return err;
 	}
 	return kSCErr_None;
 }
@@ -833,6 +835,8 @@ SCErr meth_s_new(World *inWorld, int inSize, char *inData, ReplyAddress* /*inRep
 		case 4 : {
 			Node *replaceThisNode = Msg_GetNode(inWorld, msg);
 			if (!replaceThisNode) return kSCErr_NodeNotFound;
+			Node_RemoveID(replaceThisNode);
+
 			err = Graph_New(inWorld, def, nodeID, &msg, &graph,true);
 			if (err) return err;
 			Node_Replace(&graph->mNode, replaceThisNode);
@@ -975,6 +979,9 @@ SCErr meth_g_new(World *inWorld, int inSize, char *inData, ReplyAddress* /*inRep
 			case 4 : {
 				Node *replaceThisNode = Msg_GetNode(inWorld, msg);
 				if (!replaceThisNode) return kSCErr_TargetNodeNotFound;
+        if (replaceThisNode->mID == 0) return kSCErr_ReplaceRootGroup;
+        Node_RemoveID(replaceThisNode);
+
 				err = Group_New(inWorld, newGroupID, &newGroup);
 				if (err) return err;
 				Node_Replace(&newGroup->mNode, replaceThisNode);
@@ -1083,7 +1090,6 @@ SCErr meth_n_after(World *inWorld, int inSize, char *inData, ReplyAddress* /*inR
 SCErr meth_n_order(World *inWorld, int inSize, char *inData, ReplyAddress *inReply);
 SCErr meth_n_order(World *inWorld, int inSize, char *inData, ReplyAddress* /*inReply*/)
 {
-	SCErr err;
 
 	Node *prevNode = 0;
 	Node *node = 0;
@@ -1738,7 +1744,7 @@ SCErr meth_s_getn(World *inWorld, int inSize, char *inData, ReplyAddress* inRepl
 
 	// figure out how many tags to allocate
 	int numcontrols = 0;
-	int numheads = msg.tags ? strlen(msg.tags) - 1 >> 1 : msg.remain() >> 3;
+	int numheads = msg.tags ? (strlen(msg.tags) - 1) >> 1 : msg.remain() >> 3;
 
 	while (msg.remain()) {
 		msg.geti(); // skip start
@@ -1807,7 +1813,7 @@ SCErr meth_s_noid(World *inWorld, int inSize, char *inData, ReplyAddress* inRepl
 		Graph *graph = Msg_GetGraph(inWorld, msg);
 		if (!graph) continue;
 
-		Graph_RemoveID(inWorld, graph);
+		Node_RemoveID(&graph->mNode);
 	}
 
 	return kSCErr_None;
@@ -1876,9 +1882,9 @@ void initMiscCommands()
 	//NEW_COMMAND(n_cmd);
 	NEW_COMMAND(n_map);
 	NEW_COMMAND(n_mapn);
-        NEW_COMMAND(n_mapa);
-        NEW_COMMAND(n_mapan);
-        NEW_COMMAND(n_set);
+	NEW_COMMAND(n_mapa);
+	NEW_COMMAND(n_mapan);
+	NEW_COMMAND(n_set);
 	NEW_COMMAND(n_setn);
 	NEW_COMMAND(n_fill);
 
@@ -1934,5 +1940,3 @@ void initMiscCommands()
 
 	NEW_COMMAND(error);
 }
-
-
