@@ -18,6 +18,14 @@
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
+#ifdef _WIN32
+#include "SC_Win32Utils.h"
+#endif
+
+#ifdef __APPLE__
+#include "../../common/SC_Apple.hpp"
+#endif
+
 
 #include "SC_World.h"
 #include "SC_WorldOptions.h"
@@ -33,14 +41,11 @@
 #include "SC_Errors.h"
 #include <stdio.h>
 #include "SC_Prototypes.h"
-#include "SC_Samp.h"
 #include "SC_DirUtils.h"
-#ifdef _WIN32
-# include "../../include/server/SC_ComPort.h"
-# include "SC_Win32Utils.h"
-#else
-# include "SC_ComPort.h"
-#endif
+#include "SC_Lock.h"
+#include "SC_Lib_Cintf.h"
+#include "../../common/SC_SndFileHelpers.hpp"
+#include "../../common/Samp.hpp"
 #include "SC_StringParser.h"
 #ifdef _WIN32
 # include <direct.h>
@@ -77,16 +82,10 @@ extern "C" {
 struct SF_INFO {};
 #endif
 
-int sndfileFormatInfoFromStrings(struct SF_INFO *info,
-	const char *headerFormatString, const char *sampleFormatString);
-
 bool SendMsgToEngine(World *inWorld, FifoMsg& inMsg);
 bool SendMsgFromEngine(World *inWorld, FifoMsg& inMsg);
 }
 
-bool sc_UseVectorUnit();
-void sc_SetDenormalFlags();
-
 ////////////////////////////////////////////////////////////////////////////////
 
 inline void* sc_malloc(size_t size)
@@ -140,12 +139,12 @@ void* sc_dbg_zalloc(size_t n, size_t size, const char* tag, int line)
 }
 
 # if SC_DEBUG_MEMORY
-#  define malloc(size)			sc_dbg_malloc((size), __FUNCTION__, __LINE__)
-#  define free(ptr)				sc_dbg_free((ptr), __FUNCTION__, __LINE__)
+#  define malloc_alig(size)			sc_dbg_malloc((size), __FUNCTION__, __LINE__)
+#  define free_alig(ptr)				sc_dbg_free((ptr), __FUNCTION__, __LINE__)
 #  define zalloc_(n, size)		sc_dbg_zalloc((n), (size), __FUNCTION__, __LINE__)
 # else
-#  define malloc(size)			sc_malloc((size))
-#  define free(ptr)				sc_free((ptr))
+#  define malloc_alig(size)			sc_malloc((size))
+#  define free_alig(ptr)				sc_free((ptr))
 #  define zalloc_(n, size)		sc_zalloc((n), (size))
 # endif // SC_DEBUG_MEMORY
 
@@ -154,6 +153,41 @@ void* zalloc(size_t n, size_t size)
 	return zalloc_(n, size);
 }
 
+void zfree(void * ptr)
+{
+	return free_alig(ptr);
+}
+
+
+////////////////////////////////////////////////////////////////////////////////
+
+// Set denormal FTZ mode on CPUs that need/support it.
+void sc_SetDenormalFlags();
+
+#ifdef __SSE2__
+#include <xmmintrin.h>
+
+void sc_SetDenormalFlags()
+{
+	_MM_SET_FLUSH_ZERO_MODE(_MM_FLUSH_ZERO_ON);
+	_mm_setcsr(_mm_getcsr() | 0x40); // DAZ
+}
+
+#elif defined(__SSE__)
+#include <xmmintrin.h>
+
+void sc_SetDenormalFlags()
+{
+	_MM_SET_FLUSH_ZERO_MODE(_MM_FLUSH_ZERO_ON);
+}
+
+#else
+
+void sc_SetDenormalFlags()
+{
+}
+
+#endif
 
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -210,7 +244,7 @@ void InterfaceTable_Init()
 	ft->fNRTLock = &World_NRTLock;
 	ft->fNRTUnlock = &World_NRTUnlock;
 
-	ft->mAltivecAvailable = sc_UseVectorUnit();
+	ft->mUnused0 = false;
 
 	ft->fGroup_DeleteAll = &Group_DeleteAll;
 	ft->fDoneAction = &Unit_DoneAction;
@@ -260,7 +294,13 @@ void World_LoadGraphDefs(World* world)
 
 }
 
-SC_DLLEXPORT_C World* World_New(WorldOptions *inOptions)
+namespace scsynth {
+void startAsioThread();
+void stopAsioThread();
+}
+
+
+World* World_New(WorldOptions *inOptions)
 {
 #if (_POSIX_MEMLOCK - 0) >=  200112L
 	if (inOptions->mMemoryLocking && inOptions->mRealTime)
@@ -305,17 +345,21 @@ SC_DLLEXPORT_C World* World_New(WorldOptions *inOptions)
 		world->hw = (HiddenWorld*)zalloc(1, sizeof(HiddenWorld));
 
 		world->hw->mAllocPool = new AllocPool(malloc, free, inOptions->mRealTimeMemorySize * 1024, 0);
-		world->hw->mQuitProgram = new SC_Semaphore(0);
+		world->hw->mQuitProgram = new boost::sync::semaphore(0);
 		world->hw->mTerminating = false;
 
-		extern Malloc gMalloc;
-
 		HiddenWorld *hw = world->hw;
 		hw->mGraphDefLib = new HashTable<struct GraphDef, Malloc>(&gMalloc, inOptions->mMaxGraphDefs, false);
 		hw->mNodeLib = new IntHashTable<Node, AllocPool>(hw->mAllocPool, inOptions->mMaxNodes, false);
 		hw->mUsers = (ReplyAddress*)zalloc(inOptions->mMaxLogins, sizeof(ReplyAddress));
 		hw->mNumUsers = 0;
 		hw->mMaxUsers = inOptions->mMaxLogins;
+        hw->mClientIDs = (uint32*)zalloc(inOptions->mMaxLogins, sizeof(uint32));
+        for (int i = 0; i<hw->mMaxUsers; i++) {
+            hw->mClientIDs[i] = i;
+        }
+        hw->mClientIDTop = 0;
+        hw->mClientIDdict = new ClientIDDict();
 		hw->mHiddenID = -8;
 		hw->mRecentID = -8;
 
@@ -341,8 +385,10 @@ SC_DLLEXPORT_C World* World_New(WorldOptions *inOptions)
 			server_shared_memory_creator::cleanup(inOptions->mSharedMemoryID);
 			hw->mShmem = new server_shared_memory_creator(inOptions->mSharedMemoryID, inOptions->mNumControlBusChannels);
 			world->mControlBus = hw->mShmem->get_control_busses();
-		} else
+		} else {
+			hw->mShmem = 0;
 			world->mControlBus = (float*)zalloc(world->mNumControlBusChannels, sizeof(float));
+		}
 
 		world->mNumSharedControls = 0;
 		world->mSharedControls = inOptions->mSharedControls;
@@ -396,9 +442,6 @@ SC_DLLEXPORT_C World* World_New(WorldOptions *inOptions)
 
 		world->mRestrictedPath = inOptions->mRestrictedPath;
 
-		if(inOptions->mVerbosity >= 1) {
-			scprintf("Using vector unit: %s\n", sc_UseVectorUnit() ? "yes" : "no");
-		}
 		sc_SetDenormalFlags();
 
 		if (world->mRealTime) {
@@ -422,19 +465,27 @@ SC_DLLEXPORT_C World* World_New(WorldOptions *inOptions)
 				scprintf("start audio failed.\n");
 				return 0;
 			}
+			
+#ifdef __APPLE__
+			SC::Apple::disableAppNap();
+#endif
+			
+			
 		} else {
 			hw->mAudioDriver = 0;
 		}
+
+		scsynth::startAsioThread();
 	} catch (std::exception& exc) {
 		scprintf("Exception in World_New: %s\n", exc.what());
-		World_Cleanup(world);
+		World_Cleanup(world,true);
 		return 0;
 	} catch (...) {
 	}
 	return world;
 }
 
-SC_DLLEXPORT_C int World_CopySndBuf(World *world, uint32 index, SndBuf *outBuf, bool onlyIfChanged, bool *outDidChange)
+int World_CopySndBuf(World *world, uint32 index, SndBuf *outBuf, bool onlyIfChanged, bool *outDidChange)
 {
 	if (index > world->mNumSndBufs) return kSCErr_IndexOutOfRange;
 
@@ -444,7 +495,7 @@ SC_DLLEXPORT_C int World_CopySndBuf(World *world, uint32 index, SndBuf *outBuf,
 	if (!onlyIfChanged || didChange)
 	{
 
-		world->mNRTLock->Lock();
+		reinterpret_cast<SC_Lock*>(world->mNRTLock)->lock();
 
 		SndBuf *buf = world->mSndBufsNonRealTimeMirror + index;
 
@@ -453,8 +504,8 @@ SC_DLLEXPORT_C int World_CopySndBuf(World *world, uint32 index, SndBuf *outBuf,
 			uint32 bufSize = buf->samples * sizeof(float);
 			if (buf->samples != outBuf->samples)
 			{
-				free(outBuf->data);
-				outBuf->data = (float*)malloc(bufSize);
+				free_alig(outBuf->data);
+				outBuf->data = (float*)malloc_alig(bufSize);
 			}
 			memcpy(outBuf->data, buf->data, bufSize);
 			outBuf->channels 	= buf->channels;
@@ -465,7 +516,7 @@ SC_DLLEXPORT_C int World_CopySndBuf(World *world, uint32 index, SndBuf *outBuf,
 		}
 		else
 		{
-			free(outBuf->data);
+			free_alig(outBuf->data);
 			outBuf->data = 0;
 			outBuf->channels 	= 0;
 			outBuf->samples 	= 0;
@@ -481,7 +532,7 @@ SC_DLLEXPORT_C int World_CopySndBuf(World *world, uint32 index, SndBuf *outBuf,
 
 		updates->reads = updates->writes;
 
-		world->mNRTLock->Unlock();
+		reinterpret_cast<SC_Lock*>(world->mNRTLock)->unlock();
 	}
 
 	if (outDidChange) *outDidChange = didChange;
@@ -496,12 +547,15 @@ bool nextOSCPacket(FILE *file, OSC_Packet *packet, int64& outTime)
 		return true;
 	// msglen is in network byte order
 	msglen = OSCint((char*)&msglen);
-	if (msglen > 8192)
-		throw std::runtime_error("OSC packet too long. > 8192 bytes\n");
+	if (msglen > 1073741824){
+		throw std::runtime_error("OSC packet too long. > 2^30 bytes\n");
+	}
+        packet->mData = (char *)realloc((void *)packet->mData, (size_t)msglen);
+        if(!packet->mData) throw std::runtime_error("nextOSCPacket: realloc failed...\n");
 
 	size_t read = fread(packet->mData, 1, msglen, file);
 	if (read != msglen)
-		throw std::runtime_error("nextOSCPacket: invalid read of OSC packaet\n");
+		throw std::runtime_error("nextOSCPacket: invalid read of OSC packet\n");
 
 	if (strcmp(packet->mData, "#bundle")!=0)
 		throw std::runtime_error("OSC packet not a bundle\n");
@@ -515,7 +569,7 @@ bool nextOSCPacket(FILE *file, OSC_Packet *packet, int64& outTime)
 void PerformOSCBundle(World *inWorld, OSC_Packet *inPacket);
 
 #ifndef NO_LIBSNDFILE
-SC_DLLEXPORT_C void World_NonRealTimeSynthesis(struct World *world, WorldOptions *inOptions)
+void World_NonRealTimeSynthesis(struct World *world, WorldOptions *inOptions)
 {
 	if (inOptions->mLoadGraphDefs) {
 		World_LoadGraphDefs(world);
@@ -542,6 +596,8 @@ SC_DLLEXPORT_C void World_NonRealTimeSynthesis(struct World *world, WorldOptions
 		inOptions->mNonRealTimeOutputHeaderFormat, inOptions->mNonRealTimeOutputSampleFormat);
 
 	world->hw->mNRTOutputFile = sf_open(inOptions->mNonRealTimeOutputFilename, SFM_WRITE, &outputFileInfo);
+	sf_command(world->hw->mNRTOutputFile, SFC_SET_CLIPPING, NULL, SF_TRUE);
+
 	if (!world->hw->mNRTOutputFile)
 		throw std::runtime_error("Couldn't open non real time output file.\n");
 
@@ -577,10 +633,9 @@ SC_DLLEXPORT_C void World_NonRealTimeSynthesis(struct World *world, WorldOptions
 	if (!cmdFile)
 		throw std::runtime_error("Couldn't open non real time command file.\n");
 
-	char msgbuf[8192];
 	OSC_Packet packet;
 	memset(&packet, 0, sizeof(packet));
-	packet.mData = msgbuf;
+	packet.mData = (char *)malloc(8192);
 	packet.mIsBundle = true;
 	packet.mReplyAddr.mReplyFunc = null_reply_func;
 
@@ -652,9 +707,9 @@ SC_DLLEXPORT_C void World_NonRealTimeSynthesis(struct World *world, WorldOptions
 
 				PerformOSCBundle(world, &packet);
 				if (nextOSCPacket(cmdFile, &packet, schedTime)) { run = false; break; }
-	if(inOptions->mVerbosity >= 0) {
-        printf("nextOSCPacket %g\n", schedTime * oscToSeconds);
-	}
+				if(inOptions->mVerbosity >= 0) {
+					printf("nextOSCPacket %g\n", schedTime * oscToSeconds);
+				}
 				if (schedTime < prevTime) {
 					scprintf("ERROR: Packet time stamps out-of-order.\n");
 					run = false;
@@ -669,23 +724,23 @@ SC_DLLEXPORT_C void World_NonRealTimeSynthesis(struct World *world, WorldOptions
 			float *outBus = outputBuses;
 			for (int j=0; j<numOutputChannels; ++j, outBus += bufLength) {
 				float *outFileBufPtr = outBufPos + j;
-                                if (outputTouched[j] == bufCounter) {
-                                    for (int k=0; k<bufLength; ++k) {
-                                            *outFileBufPtr = outBus[k];
-                                            outFileBufPtr += numOutputChannels;
-                                    }
-                                } else {
-                                    for (int k=0; k<bufLength; ++k) {
-                                            *outFileBufPtr = 0.f;
-                                            outFileBufPtr += numOutputChannels;
-                                    }
-                                }
+				if (outputTouched[j] == bufCounter) {
+					for (int k=0; k<bufLength; ++k) {
+						*outFileBufPtr = outBus[k];
+						outFileBufPtr += numOutputChannels;
+					}
+				} else {
+					for (int k=0; k<bufLength; ++k) {
+						*outFileBufPtr = 0.f;
+						outFileBufPtr += numOutputChannels;
+					}
+				}
 			}
 			bufFramesCalculated += bufLength;
 			inBufPos += inBufStep;
 			outBufPos += outBufStep;
 			world->mBufCounter++;
-                        oscTime = nextTime;
+			oscTime = nextTime;
 		}
 
 Bail:
@@ -693,48 +748,25 @@ Bail:
 		sf_writef_float(world->hw->mNRTOutputFile, outputFileBuf, bufFramesCalculated);
 	}
 
-        if (cmdFile != stdin) fclose(cmdFile);
+	if (cmdFile != stdin) fclose(cmdFile);
 	sf_close(world->hw->mNRTOutputFile);
-        world->hw->mNRTOutputFile = 0;
+	world->hw->mNRTOutputFile = 0;
 
 	if (world->hw->mNRTInputFile) {
-            sf_close(world->hw->mNRTInputFile);
-            world->hw->mNRTInputFile = 0;
-        }
+		sf_close(world->hw->mNRTInputFile);
+		world->hw->mNRTInputFile = 0;
+	}
 
-	World_Cleanup(world);
+	free(packet.mData);
+	World_Cleanup(world,true);
 }
 #endif   // !NO_LIBSNDFILE
 
-SC_DLLEXPORT_C int World_OpenUDP(struct World *inWorld, int inPort)
+void World_WaitForQuit(struct World *inWorld, bool unload_plugins)
 {
 	try {
-		new SC_UdpInPort(inWorld, inPort);
-		return true;
-	} catch (std::exception& exc) {
-		scprintf("Exception in World_OpenUDP: %s\n", exc.what());
-	} catch (...) {
-	}
-	return false;
-}
-
-SC_DLLEXPORT_C int World_OpenTCP(struct World *inWorld, int inPort, int inMaxConnections, int inBacklog)
-{
-	try {
-		new SC_TcpInPort(inWorld, inPort, inMaxConnections, inBacklog);
-		return true;
-	} catch (std::exception& exc) {
-		scprintf("Exception in World_OpenTCP: %s\n", exc.what());
-	} catch (...) {
-	}
-	return false;
-}
-
-SC_DLLEXPORT_C void World_WaitForQuit(struct World *inWorld)
-{
-	try {
-		inWorld->hw->mQuitProgram->Acquire();
-		World_Cleanup(inWorld);
+		inWorld->hw->mQuitProgram->wait();
+		World_Cleanup(inWorld, unload_plugins);
 	} catch (std::exception& exc) {
 		scprintf("Exception in World_WaitForQuit: %s\n", exc.what());
 	} catch (...) {
@@ -962,7 +994,7 @@ void World_Start(World *inWorld)
 	for (uint32 i=0; i<inWorld->mNumAudioBusChannels; ++i) inWorld->mAudioBusTouched[i] = -1;
 	for (uint32 i=0; i<inWorld->mNumControlBusChannels; ++i) inWorld->mControlBusTouched[i] = -1;
 
-	inWorld->hw->mWireBufSpace = (float*)malloc(inWorld->hw->mMaxWireBufs * inWorld->mBufLength * sizeof(float));
+	inWorld->hw->mWireBufSpace = (float*)malloc_alig(inWorld->hw->mMaxWireBufs * inWorld->mBufLength * sizeof(float));
 
 	inWorld->hw->mTriggers.MakeEmpty();
 	inWorld->hw->mNodeMsgs.MakeEmpty();
@@ -970,10 +1002,15 @@ void World_Start(World *inWorld)
 	inWorld->mRunning = true;
 }
 
-SC_DLLEXPORT_C void World_Cleanup(World *world)
+void World_Cleanup(World *world, bool unload_plugins)
 {
 	if (!world) return;
 
+	scsynth::stopAsioThread();
+    
+    if(unload_plugins)
+        deinitialize_library();
+    
 	HiddenWorld *hw = world->hw;
 
 	if (hw && world->mRealTime) hw->mAudioDriver->Stop();
@@ -982,22 +1019,23 @@ SC_DLLEXPORT_C void World_Cleanup(World *world)
 
 	if (world->mTopGroup) Group_DeleteAll(world->mTopGroup);
 
-	world->mDriverLock->Lock(); // never unlock..
+	reinterpret_cast<SC_Lock*>(world->mDriverLock)->lock();
 	if (hw) {
-		free(hw->mWireBufSpace);
+		sc_free(hw->mWireBufSpace);
 		delete hw->mAudioDriver;
 		hw->mAudioDriver = 0;
 	}
-	delete world->mNRTLock;
-	delete world->mDriverLock;
+	delete reinterpret_cast<SC_Lock*>(world->mNRTLock);
+	reinterpret_cast<SC_Lock*>(world->mDriverLock)->unlock();
+	delete reinterpret_cast<SC_Lock*>(world->mDriverLock);
 	World_Free(world, world->mTopGroup);
 
 	for (uint32 i=0; i<world->mNumSndBufs; ++i) {
 		SndBuf *nrtbuf = world->mSndBufsNonRealTimeMirror + i;
 		SndBuf * rtbuf = world->mSndBufs + i;
 
-		if (nrtbuf->data) free(nrtbuf->data);
-		if (rtbuf->data && rtbuf->data != nrtbuf->data) free(rtbuf->data);
+		if (nrtbuf->data) free_alig(nrtbuf->data);
+		if (rtbuf->data && rtbuf->data != nrtbuf->data) free_alig(rtbuf->data);
 
 #ifndef NO_LIBSNDFILE
 		if (nrtbuf->sndfile) sf_close(nrtbuf->sndfile);
@@ -1005,16 +1043,16 @@ SC_DLLEXPORT_C void World_Cleanup(World *world)
 #endif
 	}
 
-	free(world->mSndBufsNonRealTimeMirror);
-	free(world->mSndBufs);
+	free_alig(world->mSndBufsNonRealTimeMirror);
+	free_alig(world->mSndBufs);
 
-	free(world->mControlBusTouched);
-	free(world->mAudioBusTouched);
+	free_alig(world->mControlBusTouched);
+	free_alig(world->mAudioBusTouched);
 	if (hw->mShmem) {
 		delete hw->mShmem;
 	} else
-		free(world->mControlBus);
-	free(world->mAudioBus);
+		free_alig(world->mControlBus);
+	free_alig(world->mAudioBus);
 	delete [] world->mRGen;
 	if (hw) {
 
@@ -1023,25 +1061,27 @@ SC_DLLEXPORT_C void World_Cleanup(World *world)
 		if (hw->mNRTOutputFile) sf_close(hw->mNRTOutputFile);
 		if (hw->mNRTCmdFile) fclose(hw->mNRTCmdFile);
 #endif
-		free(hw->mUsers);
+		free_alig(hw->mUsers);
+        free_alig(hw->mClientIDs);
+        delete hw->mClientIDdict;
 		delete hw->mNodeLib;
 		delete hw->mGraphDefLib;
 		delete hw->mQuitProgram;
 		delete hw->mAllocPool;
-		free(hw);
+		free_alig(hw);
 	}
-	free(world);
+	free_alig(world);
 }
 
 
 void World_NRTLock(World *world)
 {
-	world->mNRTLock->Lock();
+	reinterpret_cast<SC_Lock*>(world->mNRTLock)->lock();
 }
 
 void World_NRTUnlock(World *world)
 {
-	world->mNRTLock->Unlock();
+	reinterpret_cast<SC_Lock*>(world->mNRTLock)->unlock();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1105,94 +1145,6 @@ SCErr bufAlloc(SndBuf* buf, int numChannels, int numFrames, double sampleRate)
 	return kSCErr_None;
 }
 
-#ifndef NO_LIBSNDFILE
-int sampleFormatFromString(const char* name);
-int sampleFormatFromString(const char* name)
-{
-	if (!name) return SF_FORMAT_PCM_16;
-
-	size_t len = strlen(name);
-	if (len < 1) return 0;
-
-	if (name[0] == 'u') {
-		if (len < 5) return 0;
-		if (name[4] == '8') return SF_FORMAT_PCM_U8; // uint8
-		return 0;
-	} else if (name[0] == 'i') {
-		if (len < 4) return 0;
-		if (name[3] == '8') return SF_FORMAT_PCM_S8;      // int8
-		else if (name[3] == '1') return SF_FORMAT_PCM_16; // int16
-		else if (name[3] == '2') return SF_FORMAT_PCM_24; // int24
-		else if (name[3] == '3') return SF_FORMAT_PCM_32; // int32
-	} else if (name[0] == 'f') {
-		return SF_FORMAT_FLOAT; // float
-	} else if (name[0] == 'd') {
-		return SF_FORMAT_DOUBLE; // double
-	} else if (name[0] == 'm' || name[0] == 'u') {
-		return SF_FORMAT_ULAW; // mulaw ulaw
-	} else if (name[0] == 'a') {
-		return SF_FORMAT_ALAW; // alaw
-	}
-	return 0;
-}
-
-int headerFormatFromString(const char *name);
-int headerFormatFromString(const char *name)
-{
-	if (!name) return SF_FORMAT_AIFF;
-	if (strcasecmp(name, "AIFF")==0) return SF_FORMAT_AIFF;
-	if (strcasecmp(name, "AIFC")==0) return SF_FORMAT_AIFF;
-	if (strcasecmp(name, "RIFF")==0) return SF_FORMAT_WAV;
-	if (strcasecmp(name, "WAVEX")==0) return SF_FORMAT_WAVEX;
-	if (strcasecmp(name, "WAVE")==0) return SF_FORMAT_WAV;
-	if (strcasecmp(name, "WAV" )==0) return SF_FORMAT_WAV;
-	if (strcasecmp(name, "Sun" )==0) return SF_FORMAT_AU;
-	if (strcasecmp(name, "IRCAM")==0) return SF_FORMAT_IRCAM;
-	if (strcasecmp(name, "NeXT")==0) return SF_FORMAT_AU;
-	if (strcasecmp(name, "raw")==0) return SF_FORMAT_RAW;
-	if (strcasecmp(name, "MAT4")==0) return SF_FORMAT_MAT4;
-	if (strcasecmp(name, "MAT5")==0) return SF_FORMAT_MAT5;
-	if (strcasecmp(name, "PAF")==0) return SF_FORMAT_PAF;
-	if (strcasecmp(name, "SVX")==0) return SF_FORMAT_SVX;
-	if (strcasecmp(name, "NIST")==0) return SF_FORMAT_NIST;
-	if (strcasecmp(name, "VOC")==0) return SF_FORMAT_VOC;
-	if (strcasecmp(name, "W64")==0) return SF_FORMAT_W64;
-	if (strcasecmp(name, "PVF")==0) return SF_FORMAT_PVF;
-	if (strcasecmp(name, "XI")==0) return SF_FORMAT_XI;
-	if (strcasecmp(name, "HTK")==0) return SF_FORMAT_HTK;
-	if (strcasecmp(name, "SDS")==0) return SF_FORMAT_SDS;
-	if (strcasecmp(name, "AVR")==0) return SF_FORMAT_AVR;
-	if (strcasecmp(name, "SD2")==0) return SF_FORMAT_SD2;
-	if (strcasecmp(name, "FLAC")==0) return SF_FORMAT_FLAC;
-// TODO allow other platforms to know vorbis once libsndfile 1.0.18 is established
-#if defined(__APPLE__) || defined(_WIN32) || LIBSNDFILE_1018
-	if (strcasecmp(name, "vorbis")==0) return SF_FORMAT_VORBIS;
-#endif
-	if (strcasecmp(name, "CAF")==0) return SF_FORMAT_CAF;
-	if (strcasecmp(name, "RF64")==0) return SF_FORMAT_RF64;
-	return 0;
-}
-
-int sndfileFormatInfoFromStrings(struct SF_INFO *info, const char *headerFormatString, const char *sampleFormatString)
-{
-	int headerFormat = headerFormatFromString(headerFormatString);
-	if (!headerFormat) return kSCErr_Failed;
-
-	int sampleFormat = sampleFormatFromString(sampleFormatString);
-	if (!sampleFormat) return kSCErr_Failed;
-
-	info->format = (unsigned int)(headerFormat | sampleFormat);
-	return kSCErr_None;
-}
-
-#else // NO_LIBSNDFILE
-
-int sndfileFormatInfoFromStrings(struct SF_INFO *info, const char *headerFormatString, const char *sampleFormatString) {
-	return kSCErr_Failed;
-}
-
-#endif // NO_LIBSNDFILE
-
 #include "scsynthsend.h"
 
 void TriggerMsg::Perform()
@@ -1340,13 +1292,13 @@ bool SendMsgFromEngine(World *inWorld, FifoMsg& inMsg)
 	return inWorld->hw->mAudioDriver->SendMsgFromEngine(inMsg);
 }
 
-SC_DLLEXPORT_C void SetPrintFunc(PrintFunc func)
+void SetPrintFunc(PrintFunc func)
 {
 	gPrint = func;
 }
 
 
-SC_DLLEXPORT_C int scprintf(const char *fmt, ...)
+int scprintf(const char *fmt, ...)
 {
 	va_list vargs;
 	va_start(vargs, fmt);
