@@ -17,6 +17,7 @@
 //  Boston, MA 02111-1307, USA.
 
 #include <cstdarg>
+#include <random>
 
 #include "sndfile.hh"
 
@@ -32,12 +33,13 @@
 #include "../utilities/malloc_aligned.hpp"
 #include "../utilities/sized_array.hpp"
 
-#include "SC_Samp.h"
 #include "SC_Prototypes.h"
-#include "SC_Errors.h"
 #include "SC_Unit.h"
+#include "SC_Lock.h"
 #include "clz.h"
 #include "SC_fftlib.h"
+#include "SC_Lock.h"
+#include "../../common/Samp.hpp"
 #include "../../common/SC_SndFileHelpers.hpp"
 
 #include <boost/math/constants/constants.hpp>
@@ -58,8 +60,8 @@ spin_lock rt_pool_guard;
 
 inline Node * as_Node(server_node * node)
 {
-    if (node == NULL)
-        return NULL;
+    if (node == nullptr)
+        return nullptr;
 
     // hack!!! we only assume that the 32bit integer mID member can be accessed via Node
     if (node->is_synth()) {
@@ -302,7 +304,7 @@ bool get_scope_buffer(World *inWorld, int index, int channels, int maxFrames, Sc
         return true;
     }
     else {
-        hnd.internalData = 0;
+        hnd.internalData = nullptr;
         return false;
     }
 }
@@ -366,7 +368,7 @@ void * rt_alloc(World * dummy, size_t size)
     if (size)
         return nova::rt_pool.malloc(size);
     else
-        return NULL;
+        return nullptr;
 }
 
 void * rt_realloc(World * dummy, void * ptr, size_t size)
@@ -397,16 +399,47 @@ void nrt_free(void * ptr)
     free(ptr);
 }
 
+void * nrt_alloc_2(World * dummy, size_t size)
+{
+    return malloc(size);
+}
+
+void * nrt_realloc_2(World * dummy, void * ptr, size_t size)
+{
+    return realloc(ptr, size);
+}
+
+void nrt_free_2(World * dummy, void * ptr)
+{
+    free(ptr);
+}
+
+
+
 void clear_outputs(Unit *unit, int samples)
 {
-    size_t outputs = unit->mNumOutputs;
+    const uint32_t outputs = unit->mNumOutputs;
 
-    if ((samples & 15) == 0)
-        for (size_t i=0; i!=outputs; ++i)
+    if( (samples & 63) == 0 ) {
+        const uint32_t loops = samples / 64;
+
+        for( int i=0; i != outputs; ++i ) {
+            float * buffer = unit->mOutBuf[i];
+            for( int loop = 0; loop != loops; ++loop ) {
+                nova::zerovec_simd<64>( buffer + loop * 64 );
+            }
+        }
+        return;
+    }
+
+    if( (samples & 15) == 0 ) {
+        for( int i=0; i != outputs; ++i )
             nova::zerovec_simd(unit->mOutBuf[i], samples);
-    else
-        for (size_t i=0; i!=outputs; ++i)
-            nova::zerovec(unit->mOutBuf[i], samples);
+        return;
+    }
+
+    for( int i=0; i != outputs; ++i )
+      nova::zerovec(unit->mOutBuf[i], samples);
 }
 
 void node_end(struct Node * node)
@@ -525,7 +558,13 @@ void done_action(int done_action, struct Unit *unit)
 
 int buf_alloc(SndBuf * buf, int channels, int frames, double samplerate)
 {
-    return nova::sc_factory->allocate_buffer(buf, channels, frames, samplerate);
+    try {
+        nova::sc_factory->allocate_buffer(buf, channels, frames, samplerate);
+        return kSCErr_None;
+    } catch(std::exception const & e) {
+        std::cout << e.what() << std::endl;
+        return kSCErr_Failed;
+    }
 }
 
 void send_trigger(Node * unit, int trigger_id, float value)
@@ -535,12 +574,12 @@ void send_trigger(Node * unit, int trigger_id, float value)
 
 void world_lock(World *world)
 {
-    world->mNRTLock->Lock();
+    reinterpret_cast<SC_Lock*>(world->mNRTLock)->lock();
 }
 
 void world_unlock(World *world)
 {
-    world->mNRTLock->Unlock();
+    reinterpret_cast<SC_Lock*>(world->mNRTLock)->unlock();
 }
 
 Node * get_node(World *world, int id)
@@ -599,6 +638,8 @@ inline void initialize_rate(Rate & rate, double sample_rate, int blocksize)
 
 void sc_plugin_interface::initialize(server_arguments const & args, float * control_busses)
 {
+    const bool nrt_mode = args.non_rt;
+
     done_nodes.reserve(64);
     pause_nodes.reserve(16);
     resume_nodes.reserve(16);
@@ -620,7 +661,7 @@ void sc_plugin_interface::initialize(server_arguments const & args, float * cont
 
     /* sndfile functions */
 #ifdef NO_LIBSNDFILE
-    sc_interface.fSndFileFormatInfoFromStrings = NULL;
+    sc_interface.fSndFileFormatInfoFromStrings = nullptr;
 #else
     sc_interface.fSndFileFormatInfoFromStrings = &sndfileFormatInfoFromStrings;
 #endif
@@ -632,9 +673,15 @@ void sc_plugin_interface::initialize(server_arguments const & args, float * cont
     sc_interface.mSineWavetable = gSineWavetable;
 
     /* memory allocation */
-    sc_interface.fRTAlloc = &rt_alloc;
-    sc_interface.fRTRealloc = &rt_realloc;
-    sc_interface.fRTFree = &rt_free;
+    if (!nrt_mode) {
+        sc_interface.fRTAlloc = &rt_alloc;
+        sc_interface.fRTRealloc = &rt_realloc;
+        sc_interface.fRTFree = &rt_free;
+    } else {
+        sc_interface.fRTAlloc = &nrt_alloc_2;
+        sc_interface.fRTRealloc = &nrt_realloc_2;
+        sc_interface.fRTFree = &nrt_free_2;
+    }
 
     sc_interface.fNRTAlloc = &nrt_alloc;
     sc_interface.fNRTRealloc = &nrt_realloc;
@@ -709,6 +756,7 @@ void sc_plugin_interface::initialize(server_arguments const & args, float * cont
     world.mNumOutputs = args.output_channels;
 
     world.mRealTime = !args.non_rt;
+    world.mVerbosity = args.verbosity;
 
     /* rngs */
     world.mNumRGens = args.rng_count;
@@ -741,7 +789,7 @@ void sc_plugin_interface::reset_sampling_rate(int sr)
 }
 
 
-void sc_done_action_handler::update_nodegraph(void)
+void sc_done_action_handler::apply_done_actions(void)
 {
     for (server_node * node : done_nodes)
         instance->free_node(node);
@@ -773,7 +821,7 @@ sc_plugin_interface::~sc_plugin_interface(void)
     delete[] world.mSndBufsNonRealTimeMirror;
     delete[] world.mSndBufUpdates;
     delete[] world.mRGen;
-    delete world.mNRTLock;
+    delete reinterpret_cast<SC_Lock*>(world.mNRTLock);
 }
 
 namespace {
@@ -787,11 +835,6 @@ sample * allocate_buffer(size_t samples)
     return ret;
 }
 
-void free_buffer(sample * chunk)
-{
-    free_aligned(chunk);
-}
-
 inline int32 bufmask(int32 x)
 {
     return (1 << (31 - CLZ(x))) - 1;
@@ -801,14 +844,14 @@ inline void sndbuf_init(SndBuf * buf)
 {
     buf->samplerate = 0;
     buf->sampledur = 0;
-    buf->data = 0;
+    buf->data = nullptr;
     buf->channels = 0;
     buf->samples = 0;
     buf->frames = 0;
     buf->mask = 0;
     buf->mask1 = 0;
     buf->coord = 0;
-    buf->sndfile = 0;
+    buf->sndfile = nullptr;
     buf->isLocal = false;
 }
 
@@ -870,27 +913,25 @@ void read_channel(SndfileHandle & sf, uint32_t channel_count, const uint32_t * c
 
 } /* namespace */
 
-int sc_plugin_interface::allocate_buffer(SndBuf * buf, uint32_t frames, uint32_t channels, double samplerate)
+void sc_plugin_interface::allocate_buffer(SndBuf * buf, uint32_t frames, uint32_t channels, double samplerate)
 {
     const uint32_t samples = frames * channels;
     if (samples == 0)
-        return kSCErr_Failed; /* invalid buffer size */
+        throw std::runtime_error( "invalid buffer size" );
 
     sample * data = nova::allocate_buffer(samples);
-    if (data == NULL)
-        return kSCErr_Failed; /* could not allocate memory */
+    if (data == nullptr)
+        throw std::runtime_error( "could not allocate memory" );
 
-    buf->data = data;
-
-    buf->channels = channels;
-    buf->frames = frames;
-    buf->samples = samples;
-    buf->mask = bufmask(samples); /* for delay lines */
-    buf->mask1 = buf->mask - 1;    /* for oscillators */
+    buf->data       = data;
+    buf->channels   = channels;
+    buf->frames     = frames;
+    buf->samples    = samples;
+    buf->mask       = bufmask(samples); /* for delay lines */
+    buf->mask1      = buf->mask - 1;    /* for oscillators */
     buf->samplerate = samplerate;
-    buf->sampledur = 1.0 / samplerate;
-    buf->isLocal = false;
-    return kSCErr_None;
+    buf->sampledur  = 1.0 / samplerate;
+    buf->isLocal    = false;
 }
 
 SndBuf * sc_plugin_interface::allocate_buffer(uint32_t index, uint32_t frames, uint32_t channels)
@@ -1049,10 +1090,10 @@ void sc_plugin_interface::buffer_close(uint32_t index)
 {
     SndBuf * buf = World_GetNRTBuf(&world, index);
 
-    if (buf->sndfile == NULL)
+    if (buf->sndfile == nullptr)
         return;
     sf_close(buf->sndfile);
-    buf->sndfile = NULL;
+    buf->sndfile = nullptr;
 }
 
 
@@ -1085,20 +1126,8 @@ void sc_plugin_interface::free_buffer(uint32_t index)
     SndBuf * buf = world.mSndBufsNonRealTimeMirror + index;
     if (buf->sndfile)
         sf_close(buf->sndfile);
+
     sndbuf_init(buf);
 }
 
-void sc_plugin_interface::initialize_synths_perform(void)
-{
-    for (std::size_t i = 0; i != uninitialized_synths.size(); ++i)
-    {
-        sc_synth * synth = static_cast<sc_synth*>(uninitialized_synths[i]);
-        if (likely(synth->get_parent())) // it is possible to remove a synth before it is initialized
-            synth->prepare();
-        synth->release();
-    }
-    synths_to_initialize = false;
-    uninitialized_synths.clear();
-}
-
 } /* namespace nova */
