@@ -32,8 +32,6 @@
 
 #include <boost/scoped_array.hpp>
 
-#include "../../common/SC_SndFileHelpers.hpp"
-
 namespace nova {
 
 class sc_done_action_handler
@@ -76,7 +74,7 @@ public:
         freeAll_nodes.push_back(node);
     }
 
-    void update_nodegraph(void);
+    void apply_done_actions(void);
 
 protected:
     typedef rt_pool_allocator<server_node*> server_node_alloc;
@@ -96,9 +94,7 @@ public:
     void initialize(class server_arguments const & args, float * control_busses);
     void reset_sampling_rate(int sr);
 
-    sc_plugin_interface(void):
-        synths_to_initialize(false)
-    {}
+    sc_plugin_interface(void) = default;
 
     ~sc_plugin_interface(void);
 
@@ -115,7 +111,7 @@ public:
     /* @{ */
     /* audio buffer handling */
     SndBuf* allocate_buffer(uint32_t index, uint32_t frames, uint32_t channels);
-    int allocate_buffer(SndBuf * buf, uint32_t frames, uint32_t channels, double samplerate);
+    void allocate_buffer(SndBuf * buf, uint32_t frames, uint32_t channels, double samplerate);
     void buffer_read_alloc(uint32_t index, const char * filename, uint32_t start, uint32_t frames);
     void buffer_alloc_read_channels(uint32_t index, const char * filename, uint32_t start, uint32_t frames, uint32_t channel_count,
                                     const uint32_t * channel_data);
@@ -161,6 +157,11 @@ public:
         return async_buffer_guards[index];
     }
 
+    bool is_realtime_synthesis() const
+    {
+        return world.mRealTime;
+    }
+
 private:
     boost::scoped_array<std::mutex> async_buffer_guards;
     /* @} */
@@ -214,30 +215,6 @@ public:
             r_values[i] = world.mControlBus[i];
     }
     /* @}*/
-
-    /* @{ */
-    /** synth initialization. called in the beginning of each dsp tick */
-    void initialize_synths(void)
-    {
-        if (likely(!synths_to_initialize))
-            return; // fast-path
-
-        initialize_synths_perform();
-    }
-
-    void schedule_for_preparation(abstract_synth * synth)
-    {
-        synths_to_initialize = true;
-        uninitialized_synths.push_back(synth);
-        synth->add_ref();
-    }
-
-private:
-    bool synths_to_initialize;
-
-    void initialize_synths_perform(void);
-    std::vector<abstract_synth*, rt_pool_allocator<abstract_synth*> > uninitialized_synths;
-    /* @} */
 };
 
 } /* namespace nova */
