@@ -36,15 +36,11 @@ namespace nova {
 
 class sc_synthdef
 {
-    typedef c_string string;
-    typedef std::vector<float, aligned_allocator<float> > fvector;
-    typedef std::vector<c_string, aligned_allocator<string> > svector;
-
+    typedef std::vector<float, aligned_allocator<float> > float_vector;
     typedef std::vector<char, aligned_allocator<char> > char_vector;
+    typedef std::map<symbol, int32_t, std::less<symbol>, aligned_allocator<std::pair<symbol, int32_t>>> parameter_index_map_t;
 
 public:
-    typedef std::map<string, int32_t, std::less<string>, aligned_allocator<string> > parameter_map_t;
-
     struct input_spec
     {
         input_spec(int32_t source, int32_t index):
@@ -63,29 +59,25 @@ public:
         int32_t source;   /* index of ugen or -1 for constant */
         int32_t index;    /* number of output or constant index */
     };
+
     typedef std::vector<input_spec, aligned_allocator<input_spec> > input_spec_vector;
 
     struct unit_spec_t
     {
-        explicit unit_spec_t(const char *& buffer, int version);
+        explicit unit_spec_t(const char *& buffer, const char * buffer_end, int version);
 
-        unit_spec_t(string const & name, int16_t rate, int16_t special_index,
+        unit_spec_t(symbol const & name, int16_t rate, int16_t special_index,
                     input_spec_vector const & in_specs,
                     char_vector const & out_specs):
             name(name), rate(rate), special_index(special_index),
             input_specs(in_specs), output_specs(out_specs)
         {}
 
-        unit_spec_t(unit_spec_t && rhs):
-            name(std::move(rhs.name)), rate(rhs.rate), special_index(rhs.special_index),
-            input_specs(std::move(rhs.input_specs)), output_specs(std::move(rhs.output_specs)),
-            buffer_mapping(std::move(rhs.buffer_mapping)), prototype(rhs.prototype)
-        {}
-
-        unit_spec_t(unit_spec_t const & rhs) = default;
+        unit_spec_t(unit_spec_t && rhs)                  = default;
+        unit_spec_t(unit_spec_t const & rhs)             = default;
         unit_spec_t & operator=(unit_spec_t const & rhs) = default;
 
-        string name;
+        symbol name;
         int16_t rate;           /* 0: scalar rate, 1: buffer rate, 2: full rate, 3: demand rate */
         int16_t special_index;
 
@@ -93,7 +85,7 @@ public:
         char_vector output_specs;      /* calculation rates */
         std::vector<int32_t, aligned_allocator<int32_t> > buffer_mapping;
 
-        std::size_t memory_requirement(void)
+        std::size_t memory_requirement(void) const
         {
             return input_specs.size()  * (sizeof(Wire*) + sizeof(float*)) +
                    output_specs.size() * (sizeof(Wire*) + sizeof(float*)) +
@@ -111,20 +103,15 @@ public:
     typedef std::vector<unit_spec_t, aligned_allocator<unit_spec_t> > graph_t;
     typedef std::vector<int32_t, aligned_allocator<int32_t> > calc_units_t;
 
-    sc_synthdef(const char *& buffer, int version);
+    sc_synthdef(const char *& buffer, const char * buffer_end, int version);
 
-    sc_synthdef(sc_synthdef && rhs):
-        name_(std::move(rhs.name_)), constants(std::move(rhs.constants)), parameters(std::move(rhs.parameters)),
-        parameter_map(std::move(rhs.parameter_map)), graph(std::move(rhs.graph)), buffer_count(rhs.buffer_count),
-        calc_unit_indices(std::move(rhs.calc_unit_indices)), memory_requirement_(rhs.memory_requirement_)
-    {}
-
-    sc_synthdef(sc_synthdef const & rhs) = default;
+    sc_synthdef(sc_synthdef && rhs)                 = default;
+    sc_synthdef(sc_synthdef const & rhs)            = default;
     sc_synthdef& operator=(sc_synthdef const & rhs) = default;
 
     std::string dump(void) const;
 
-    string const & name(void) const
+    symbol const & name(void) const
     {
         return name_;
     }
@@ -151,23 +138,23 @@ public:
     }
 
 private:
-    void read_synthdef(const char *&, int version);
+    void read_synthdef(const char *& data, const char * end, int version);
 
     /** assign buffers, collect memory requirement & cache ugen prototype */
     void prepare(void);
 
-    string name_;
-    fvector constants;
-    fvector parameters;
-    parameter_map_t parameter_map;
+    symbol name_;
+    float_vector constants;
+    float_vector parameters;
+    parameter_index_map_t parameter_map;
 
     graph_t graph;
-    std::uint16_t buffer_count;
+    unsigned int buffer_count;
     calc_units_t calc_unit_indices; /**< indices of the units, that need to be calculated */
     std::size_t memory_requirement_;
 };
 
-std::vector<sc_synthdef> read_synthdefs(const char * buf_ptr);
+std::vector<sc_synthdef> read_synthdefs(const char * buffer, const char * buffer_end);
 std::vector<sc_synthdef> read_synthdef_file(boost::filesystem::path const & filename);
 
 } /* namespace nova */
