@@ -28,6 +28,8 @@
 #include "server.hpp"
 #include "server_args.hpp"
 
+#include "SC_Version.hpp"
+
 #include "../sc/sc_ugen_factory.hpp"
 #include "../sc/sc_synth_definition.hpp"
 #include "../utilities/utils.hpp"
@@ -43,6 +45,11 @@
 #endif
 
 #include "SC_DirUtils.h"
+#ifndef _WIN32
+    const char pathSeparator[] = ":";
+#else
+    const char pathSeparator[] = ";";
+#endif
 
 using namespace nova;
 using namespace std;
@@ -55,7 +62,7 @@ void terminate(int i)
     instance->terminate();
 }
 
-void register_handles(void)
+void register_signal_handler(void)
 {
     void (*prev_fn)(int);
 
@@ -101,9 +108,9 @@ void get_jack_names(server_arguments const & args, string & server_name, string
 {
     client_name = "supernova";
 
-    if (!args.hw_name.empty()) {
+    if (!args.hw_name.empty() && !args.hw_name[0].empty()) {
         vector<string> names;
-        boost::split(names, args.hw_name, boost::algorithm::is_any_of(":"));
+        boost::split(names, args.hw_name[0], boost::algorithm::is_any_of(":"));
 
         if (names.size() == 1) {
             server_name = names[0];
@@ -142,14 +149,48 @@ void start_audio_backend(server_arguments const & args)
 
 void start_audio_backend(server_arguments const & args)
 {
-    bool success = instance->open_stream(args.hw_name, args.input_channels, args.hw_name, args.output_channels,
-        args.samplerate, args.blocksize, args.blocksize);
+    int input_channels = args.input_channels;
+    int output_channels = args.output_channels;
+
+    std::string input_device, output_device;
+    if (args.hw_name.empty()) {
+        boost::tie(input_device, output_device) = instance->default_device_names();
+    } else if (args.hw_name.size() == 1) {
+        input_device = output_device = args.hw_name[0];
+    } else {
+        input_device = args.hw_name[0];
+        output_device = args.hw_name[1];
+    }
+
+    cout << "opening portaudio device name: ";
+    cout << input_device << " / " << output_device << endl;
+
+    if (input_device == "nil") {
+        input_device.clear();
+        input_channels = 0;
+    }
+
+    if (output_device == "nil") {
+        output_device.clear();
+        output_channels = 0;
+    }
+
+    cout << "opening portaudio device name: ";
+    cout << input_device << " / " << output_device << endl;
+
+
+    bool success = instance->open_stream(input_device, input_channels, output_device, output_channels,
+        args.samplerate, args.blocksize, args.hardware_buffer_size);
 
     if (!success) {
-        cout << "could not open portaudio device name:" << args.hw_name << endl;
+        cout << "could not open portaudio device name: " << input_device << " / " << output_device << endl;
         exit(1);
     }
-    cout << "opened portaudio device name:" << args.hw_name << endl;
+    cout << "opened portaudio device name: ";
+    cout << input_device << " / " << output_device << endl;
+
+    instance->report_latency();
+
     instance->prepare_backend();
     instance->activate_audio();
 }
@@ -183,32 +224,37 @@ boost::filesystem::path resolve_home(void)
 #endif
 }
 
-void set_plugin_paths(void)
+void set_plugin_paths(server_arguments const & args, nova::sc_ugen_factory * factory)
 {
-    server_arguments const & args = server_arguments::instance();
-
     if (!args.ugen_paths.empty()) {
-        for(string const & path : args.ugen_paths)
-            sc_factory->load_plugin_folder(path);
+        for(path const & path1 : args.ugen_paths){
+            vector<std::string> directories;
+            boost::split(directories, path1.string(), boost::is_any_of(pathSeparator));
+            for(string const & path : directories){
+                factory->load_plugin_folder(path);
+            }
+        }
     } else {
 #ifdef __linux__
-        path home = resolve_home();
-        sc_factory->load_plugin_folder("/usr/local/lib/SuperCollider/plugins");
-        sc_factory->load_plugin_folder("/usr/lib/SuperCollider/plugins");
-        sc_factory->load_plugin_folder(home / "/.local/share/SuperCollider/plugins");
-        sc_factory->load_plugin_folder(home / "share/SuperCollider/plugins");
+        const path home = resolve_home();
+        std::vector<path> folders = { "/usr/local/lib/SuperCollider/plugins",
+                                      "/usr/lib/SuperCollider/plugins",
+                                      home / "/.local/share/SuperCollider/Extensions",
+                                      home / "share/SuperCollider/plugins" };
+
+        for (path const & folder : folders)
+            factory->load_plugin_folder(folder);
 #else
         char plugin_dir[MAXPATHLEN];
         sc_GetResourceDirectory(plugin_dir, MAXPATHLEN);
-        sc_AppendToPath(plugin_dir, MAXPATHLEN, "plugins");
+        factory->load_plugin_folder(path(plugin_dir) / "plugins");
 
         char extension_dir[MAXPATHLEN];
-
         sc_GetSystemExtensionDirectory(extension_dir, MAXPATHLEN);
-        sc_factory->load_plugin_folder(path(extension_dir) / "plugins");
+        factory->load_plugin_folder(path(extension_dir) / "plugins");
 
         sc_GetUserExtensionDirectory(extension_dir, MAXPATHLEN);
-        sc_factory->load_plugin_folder(path(extension_dir) / "plugins");
+        factory->load_plugin_folder(path(extension_dir) / "plugins");
 #endif
     }
 
@@ -227,10 +273,10 @@ void load_synthdef_folder(nova_server & server, path const & folder, bool verbos
 
 void load_synthdefs(nova_server & server, server_arguments const & args)
 {
-#ifndef _WIN32
-    const char pathSeparator[] = ":";
-#else
-    const char pathSeparator[] = ";";
+    using namespace std;
+
+#ifndef NDEBUG
+    auto start_time = std::chrono::high_resolution_clock::now();
 #endif
 
     if (args.load_synthdefs) {
@@ -252,7 +298,11 @@ void load_synthdefs(nova_server & server, server_arguments const & args)
             load_synthdef_folder(server, directory, args.verbosity > 0);
     }
 #ifndef NDEBUG
-    cout << "SynthDefs loaded" << endl;
+    auto end_time = std::chrono::high_resolution_clock::now();
+    cout << "SynthDefs loaded in "
+         << std::chrono::duration_cast<std::chrono::milliseconds>(end_time - start_time).count()
+         << " ms"
+         << endl;
 #endif
 }
 
@@ -260,7 +310,7 @@ void drop_rt_scheduling()
 {
     bool success = nova::thread_set_priority(0);
     if (!success)
-        cerr << "Warning: cannot drop rt priority" << endl;
+        cout << "Warning: cannot drop rt priority" << endl;
 }
 
 void enable_core_dumps(void)
@@ -271,6 +321,33 @@ void enable_core_dumps(void)
 #endif
 }
 
+void lock_memory(server_arguments const & args)
+{
+#if (_POSIX_MEMLOCK - 0) >=  200112L
+    if (args.memory_locking) {
+        bool lock_memory = false;
+
+        rlimit limit;
+
+        int failure = getrlimit(RLIMIT_MEMLOCK, &limit);
+        if (failure)
+            printf("getrlimit failure\n");
+        else {
+            if (limit.rlim_cur == RLIM_INFINITY and
+                limit.rlim_max == RLIM_INFINITY)
+                lock_memory = true;
+            else
+                printf("memory locking disabled due to resource limiting\n");
+
+            if (lock_memory) {
+                if (mlockall(MCL_FUTURE) != -1)
+                    printf("memory locking enabled.\n");
+            }
+        }
+    }
+#endif
+}
+
 } /* namespace */
 
 int main(int argc, char * argv[])
@@ -281,35 +358,13 @@ int main(int argc, char * argv[])
     server_arguments::initialize(argc, argv);
     server_arguments const & args = server_arguments::instance();
 
-#if (_POSIX_MEMLOCK - 0) >=  200112L
-    if (args.memory_locking)
-    {
-        bool lock_memory = false;
-
-        rlimit limit;
-
-        int failure = getrlimit(RLIMIT_MEMLOCK, &limit);
-        if (failure)
-            printf("getrlimit failure\n");
-        else
-        {
-            if (limit.rlim_cur == RLIM_INFINITY and
-                limit.rlim_max == RLIM_INFINITY)
-                lock_memory = true;
-            else
-                printf("memory locking disabled due to resource limiting\n");
-
-            if (lock_memory)
-            {
-                if (mlockall(MCL_FUTURE) != -1)
-                    printf("memory locking enabled.\n");
-            }
-        }
+    if(args.dump_version){
+        cout << "supernova " << SC_VersionString() << endl;
+        return 0;
     }
-#endif
-
 
     rt_pool.init(args.rt_pool_size * 1024, args.memory_locking);
+    lock_memory(args);
 
     cout << "Supernova booting" << endl;
 #ifndef NDEBUG
@@ -318,9 +373,9 @@ int main(int argc, char * argv[])
 
     server_shared_memory_creator::cleanup(args.port());
     nova_server server(args);
-    register_handles();
+    register_signal_handler();
 
-    set_plugin_paths();
+    set_plugin_paths(args, sc_factory);
     load_synthdefs(server, args);
 
     if (!args.non_rt) {
