@@ -39,23 +39,25 @@
 namespace nova
 {
 
-class nova_server * instance = 0;
+class nova_server * instance = nullptr;
 
 nova_server::nova_server(server_arguments const & args):
     server_shared_memory_creator(args.port(), args.control_busses),
-    scheduler<nova::scheduler_hook, thread_init_functor>(args.threads, !args.non_rt),
-    buffer_manager(args.buffers), sc_osc_handler(args), dsp_queue_dirty(false)
+    scheduler<thread_init_functor>(args.threads, !args.non_rt),
+    buffer_manager(args.buffers), sc_osc_handler(args)
 {
     assert(instance == 0);
     instance = this;
-
+    
+    use_system_clock = (args.use_system_clock == 1);
+    smooth_samplerate = args.samplerate;
+    
     if (!args.non_rt)
         io_interpreter.start_thread();
 
     sc_factory = new sc_ugen_factory;
     sc_factory->initialize(args, server_shared_memory_creator::shm->get_control_busses());
 
-
     /** first guess: needs to be updated, once the backend is started */
     time_per_tick = time_tag::from_samples(args.blocksize, args.samplerate);
 
@@ -81,62 +83,63 @@ void nova_server::prepare_backend(void)
 
     audio_backend::output_mapping(outputs.begin(), outputs.end());
 
-#ifdef __SSE__
-    /* denormal handling */
-    _MM_SET_FLUSH_ZERO_MODE(_MM_FLUSH_ZERO_ON);
-    _mm_setcsr(_mm_getcsr() | 0x40);
-#endif
-
     time_per_tick = time_tag::from_samples(blocksize, get_samplerate());
 }
 
 nova_server::~nova_server(void)
 {
+    //we should delete but get chrashes at the moment on linux and macosx
+    //delete sc_factory;
 #if defined(JACK_BACKEND) || defined(PORTAUDIO_BACKEND)
     deactivate_audio();
 #endif
 
-    scheduler<scheduler_hook, thread_init_functor>::terminate();
+    group_free_all(static_cast<abstract_group*>(find_node(0))); // free the root group
+
+    scheduler<thread_init_functor>::terminate();
     io_interpreter.join_thread();
-    instance = 0;
+    instance = nullptr;
+}
+
+void nova_server::perform_node_add(server_node *node, node_position_constraint const & constraints, bool update_dsp_queue)
+{
+    node_graph::add_node(node, constraints, [&](server_node & node) {
+        finalize_node(node);
+    });
+
+    if (constraints.second == replace || update_dsp_queue)
+        request_dsp_queue_update();
+
+    notification_node_started(node);
 }
 
 abstract_synth * nova_server::add_synth(const char * name, int id, node_position_constraint const & constraints)
 {
     abstract_synth * ret = synth_factory::create_instance(name, id);
+    if (ret == nullptr)
+        return nullptr;
 
-    if (ret == 0)
-        return 0;
-
-    if (constraints.second == replace)
-        notification_node_ended(constraints.first);
-
-    node_graph::add_node(ret, constraints);
-    update_dsp_queue();
-    notification_node_started(ret);
+    perform_node_add(ret, constraints, true);
     return ret;
 }
 
 group * nova_server::add_group(int id, node_position_constraint const & constraints)
 {
     group * g = new group(id);
-    if (g == 0)
-        return 0;
+    if (g == nullptr)
+        return nullptr;
 
-    instance->add_node(g, constraints);
-    /* no need to update the dsp queue */
-    notification_node_started(g);
+    perform_node_add(g, constraints, false);
     return g;
 }
 
 parallel_group * nova_server::add_parallel_group(int id, node_position_constraint const & constraints)
 {
     parallel_group * g = new parallel_group(id);
-    if (g == 0)
-        return 0;
-    instance->add_node(g, constraints);
-    /* no need to update the dsp queue */
-    notification_node_started(g);
+    if (g == nullptr)
+        return nullptr;
+
+    perform_node_add(g, constraints, false);
     return g;
 }
 
@@ -155,15 +158,51 @@ void nova_server::set_node_slot(int node_id, const char * slot, float value)
         node->set(slot, value);
 }
 
+void nova_server::finalize_node(server_node & node)
+{
+    if (node.is_synth()) {
+        sc_synth & synth = static_cast<sc_synth&>(node);
+        synth.finalize();
+    }
+    notification_node_ended(&node);
+}
+
+
 void nova_server::free_node(server_node * node)
 {
-    if (node->get_parent() == NULL)
+    if (node->get_parent() == nullptr)
         return; // has already been freed by a different event
-    notification_node_ended(node);
-    node_graph::remove_node(node);
-    update_dsp_queue();
+
+    node_graph::remove_node(node, [&] (server_node & node) {
+        finalize_node(node);
+    });
+    request_dsp_queue_update();
 }
 
+void nova_server::group_free_all(abstract_group * group)
+{
+    std::vector<server_node *, rt_pool_allocator<server_node*>> nodes_to_free;
+
+    group->apply_on_children( [&](server_node & node) {
+        nodes_to_free.push_back(&node);
+    });
+
+    for (server_node * node: nodes_to_free)
+        free_node(node);
+}
+
+void nova_server::group_free_deep(abstract_group * group)
+{
+    std::vector<server_node *, rt_pool_allocator<server_node*>> nodes_to_free;
+    group->apply_deep_on_children( [&](server_node & node) {
+        if (node.is_synth())
+            nodes_to_free.push_back(&node);
+    });
+    for (server_node * node: nodes_to_free)
+        free_node(node);
+}
+
+
 void nova_server::run_nonrt_synthesis(server_arguments const & args)
 {
     start_dsp_threads();
@@ -175,7 +214,7 @@ void nova_server::rebuild_dsp_queue(void)
 {
     assert(dsp_queue_dirty);
     node_graph::dsp_thread_queue_ptr new_queue = node_graph::generate_dsp_queue();
-    scheduler<scheduler_hook, thread_init_functor>::reset_queue_sync(std::move(new_queue));
+    scheduler<thread_init_functor>::reset_queue_sync(std::move(new_queue));
     dsp_queue_dirty = false;
 }
 
@@ -186,18 +225,54 @@ static void name_current_thread(int thread_index)
     name_thread(buf);
 }
 
-void thread_init_functor::operator()(int thread_index)
+static void set_daz_ftz(void)
 {
-    name_current_thread(thread_index);
+#ifdef __SSE__
+    /* denormal handling */
+    _MM_SET_FLUSH_ZERO_MODE(_MM_FLUSH_ZERO_ON);
+    _mm_setcsr(_mm_getcsr() | 0x40);
+#endif
+}
+
+static bool set_realtime_priority(int thread_index)
+{
+    bool success = false;
+
+#ifdef NOVA_TT_PRIORITY_PERIOD_COMPUTATION_CONSTRAINT
+    double blocksize = server_arguments::instance().blocksize;
+    double samplerate = server_arguments::instance().samplerate;
+
+    double ns_per_block = 1e9 / samplerate * blocksize;
+
+#ifdef __APPLE__
+
+    success = thread_set_priority_rt(AudioConvertNanosToHostTime(ns_per_block),
+                                     AudioConvertNanosToHostTime(ns_per_block - 2),
+                                     AudioConvertNanosToHostTime(ns_per_block - 1),
+                                     false);
+
+#else
+
+    success = thread_set_priority_rt(ns_per_block, ns_per_block - 2, ns_per_block - 1, false);
+
+#endif
+
+    if (!success)
+        std::cout << "Warning: initialize deadline scheduling" << std::endl;
+
+#endif
+
+    if (!success) {
 
-    if (rt) {
-        bool success = true;
 #ifdef NOVA_TT_PRIORITY_RT
 
 #ifdef JACK_BACKEND
         int priority = instance->realtime_priority();
-        if (priority < 0)
-            success = false;
+        if (priority >= 0)
+            success = true;
+
+#elif _WIN32
+        int priority = thread_priority_interval_rt().second;
 #else
         int min, max;
         boost::tie(min, max) = thread_priority_interval_rt();
@@ -208,25 +283,27 @@ void thread_init_functor::operator()(int thread_index)
         if (success)
             success = thread_set_priority_rt(priority);
 #endif
-
-#if defined(NOVA_TT_PRIORITY_PERIOD_COMPUTATION_CONSTRAINT) && defined (__APPLE__)
-        double blocksize = server_arguments::instance().blocksize;
-        double samplerate = server_arguments::instance().samplerate;
-
-        double ns_per_block = 1e9 / samplerate * blocksize;
-
-        success = thread_set_priority_rt(AudioConvertNanosToHostTime(ns_per_block),
-                                         AudioConvertNanosToHostTime(ns_per_block - 2),
-                                         AudioConvertNanosToHostTime(ns_per_block - 1),
-                                         false);
-#endif
-
-        if (!success)
-            std::cerr << "Warning: cannot raise thread priority" << std::endl;
     }
 
+    if (!success)
+        std::cout << "Warning: cannot raise thread priority" << std::endl;
+
+    return success;
+}
+
+
+void thread_init_functor::operator()(int thread_index)
+{
+    set_daz_ftz();
+    name_current_thread(thread_index);
+
+    if (rt)
+        set_realtime_priority(thread_index);
+
+#ifndef __APPLE__
     if (!thread_set_affinity(thread_index))
-        std::cerr << "Warning: cannot set thread affinity of audio helper thread" << std::endl;
+        std::cout << "Warning: cannot set thread affinity of audio helper thread" << std::endl;
+#endif
 }
 
 void io_thread_init_functor::operator()() const
@@ -252,8 +329,24 @@ void synth_definition_deleter::dispose(synth_definition * ptr)
 
 void realtime_engine_functor::init_thread(void)
 {
+    set_daz_ftz();
+
+#ifndef __APPLE__
     if (!thread_set_affinity(0))
-        std::cerr << "Warning: cannot set thread affinity of main audio thread" << std::endl;
+        std::cout << "Warning: cannot set thread affinity of main audio thread" << std::endl;
+#endif
+
+#ifdef JACK_BACKEND
+    set_realtime_priority(0);
+#endif
+    if(instance->use_system_clock){
+        double nows = (uint64)(OSCTime(std::chrono::system_clock::now())) * kOSCtoSecs;
+        instance->mDLL.Reset(
+            sc_factory->world.mSampleRate,
+            sc_factory->world.mBufLength,
+            SC_TIME_DLL_BW,
+            nows);
+    }
 
     name_current_thread(0);
 }
