@@ -42,17 +42,11 @@ struct audio_sync_callback:
     public static_pooled_class<audio_sync_callback, 1<<20 /* 1mb pool of realtime memory */,
                                true>
 {
-    virtual ~audio_sync_callback()
-    {}
+    virtual ~audio_sync_callback() = default;
 
     virtual void run(void) = 0;
 };
 
-struct nop_hook
-{
-    void operator()(void) {}
-};
-
 /** scheduler class of the nova server
  *
  *  - provides a callback system to place callbacks in the scheduler
@@ -64,15 +58,12 @@ struct nop_hook
  *  thread_init_functor: helper thread initialization functor
  *
  * */
-template<class scheduler_hook = nop_hook,
-         class thread_init_functor = nop_thread_init
-        >
-class scheduler:
-    protected scheduler_hook
+template<class thread_init_functor = nop_thread_init>
+class scheduler
 {
-    typedef nova::dsp_threads<queue_node, thread_init_functor, rt_pool_allocator<void*> > dsp_threads;
-    typedef typename dsp_threads::dsp_thread_queue_ptr dsp_thread_queue_ptr;
-    typedef typename dsp_threads::thread_count_t thread_count_t;
+    typedef nova::dsp_thread_pool<queue_node, thread_init_functor, rt_pool_allocator<void*> > dsp_thread_pool;
+    typedef typename dsp_thread_pool::dsp_thread_queue_ptr dsp_thread_queue_ptr;
+    typedef typename dsp_thread_pool::thread_count_t thread_count_t;
 
     struct reset_queue_cb:
         public audio_sync_callback
@@ -82,7 +73,7 @@ class scheduler:
             sched(sched), qptr(qptr)
         {}
 
-        void run(void)
+        void run(void) override
         {
             sched->reset_queue_sync(qptr);
             /** todo: later free the queue in a helper thread */
@@ -103,7 +94,7 @@ protected:
 public:
     /* start thread_count - 1 scheduler threads */
     scheduler(thread_count_t thread_count = 1, bool realtime = false):
-        threads(thread_count, thread_init_functor(realtime))
+        threads(thread_count, !realtime, thread_init_functor(realtime))
     {}
 
     void start_dsp_threads(void)
@@ -125,10 +116,13 @@ public:
     }
 
     /* called from the audio driver */
-    void operator()(void)
+    void run_callbacks()
     {
         cbs.run_callbacks();
-        scheduler_hook::operator()();
+    }
+
+    void compute_audio()
+    {
         threads.run();
     }
 
@@ -140,11 +134,10 @@ public:
 
 private:
     callback_system<audio_sync_callback, false> cbs;
-    dsp_threads threads;
+    dsp_thread_pool threads;
     std::mutex sync_mutex;
 };
 
 } /* namespace nova */
 
 #endif /* SERVER_SCHEDULER_HPP */
-
