@@ -25,8 +25,6 @@
 #include <new>     // for std::bad_alloc
 #include <utility> // for std::forward
 
-#include <boost/noncopyable.hpp>
-
 #ifdef __SSE2__
 #include <xmmintrin.h>
 #elif defined(HAVE_TBB)
@@ -51,7 +49,7 @@ namespace nova {
  */
 const int malloc_memory_alignment = 64;
 
-inline void* MALLOC malloc_aligned(std::size_t nbytes)
+inline void* MALLOC ASSUME_ALIGNED(64) malloc_aligned(std::size_t nbytes)
 {
     void * ret;
     int status = posix_memalign(&ret, malloc_memory_alignment, nbytes);
@@ -66,6 +64,20 @@ inline void free_aligned(void *ptr)
     free(ptr);
 }
 
+#elif defined(__SSE2__)
+
+const int malloc_memory_alignment = 64;
+
+inline void* MALLOC ASSUME_ALIGNED(64) malloc_aligned(std::size_t nbytes)
+{
+    return _mm_malloc(nbytes, malloc_memory_alignment);
+}
+
+inline void free_aligned(void *ptr)
+{
+    _mm_free(ptr);
+}
+
 #elif defined(__APPLE__)
 
 const int malloc_memory_alignment = 64;
@@ -82,25 +94,11 @@ inline void free_aligned(void *ptr)
 }
 
 
-#elif defined(__SSE2__)
-
-const int malloc_memory_alignment = 64;
-
-inline void* MALLOC malloc_aligned(std::size_t nbytes)
-{
-    return _mm_malloc(nbytes, malloc_memory_alignment);
-}
-
-inline void free_aligned(void *ptr)
-{
-    _mm_free(ptr);
-}
-
 #elif defined(_MSC_VER)
 
 const int malloc_memory_alignment = 64;
 
-inline void* MALLOC malloc_aligned(std::size_t nbytes)
+inline void* MALLOC ASSUME_ALIGNED(64) malloc_aligned(std::size_t nbytes)
 {
     return _aligned_malloc(nbytes, malloc_memory_alignment);
 }
@@ -129,11 +127,11 @@ inline void free_aligned(void *ptr)
 /* on other systems, we use the aligned memory allocation taken
  * from thomas grill's implementation for pd */
 #define VECTORALIGNMENT 128
-inline void* MALLOC malloc_aligned(std::size_t nbytes)
+inline void* MALLOC ASSUME_ALIGNED(64) malloc_aligned(std::size_t nbytes)
 {
     void* vec = malloc(nbytes+ (VECTORALIGNMENT/8-1) + sizeof(void *));
 
-    if (vec != NULL) {
+    if (vec != nullptr) {
         /* get alignment of first possible signal vector byte */
         long alignment = ((long)vec+sizeof(void *))&(VECTORALIGNMENT/8-1);
         /* calculate aligned pointer */
@@ -147,7 +145,7 @@ inline void* MALLOC malloc_aligned(std::size_t nbytes)
 
 inline void free_aligned(void *ptr)
 {
-    if (ptr == NULL)
+    if (ptr == nullptr)
         return;
 
     /* get original memory location */
@@ -216,10 +214,10 @@ public:
         return &x;
     }
 
-    pointer allocate(size_type n, const void* hint = 0)
+    pointer allocate(size_type n, const void* hint = nullptr)
     {
         pointer ret = malloc_aligned<T>(n);
-        if (ret == 0)
+        if (ret == nullptr)
             throw std::bad_alloc();
         return ret;
     }
@@ -243,7 +241,7 @@ public:
     template< class U, class... Args >
     void construct(U * p, Args&& ... args)
     {
-        ::new(p) T(std::forward<Args>(args)...);
+        ::new((void*)p) U(std::forward<Args>(args)...);
     }
 #endif
 
@@ -276,6 +274,10 @@ public:
         ptr(p)
     {}
 
+    explicit aligned_storage_ptr(size_t count):
+        ptr(malloc_aligned<T>(count))
+    {}
+
     ~aligned_storage_ptr(void)
     {
         if (managed && ptr)
